Package: PHENTHAUproc
Title: Phenology Modelling of Thaumetopoea Processionea
Version: 1.1.2
Authors@R: c(
    person("Lorenz", "Bachfischer", email = "lorenz.bachfischer@posteo.de", role = c("aut", "cre")),
    person("Department of Forest Protection, FVA", email = "Waldschutz.FVA-BW@forst.bwl.de", role = c("fnd")))
Description: Methods to calculate and present 'PHENTHAUproc', an early warning and decision support system for hazard assessment and control of oak processionary moth (OPM) using local and spatial temperature data. It was created by Halbig et al. 2024 (<doi:10.1016/j.foreco.2023.121525>) at FVA (<https://www.fva-bw.de/en/homepage/>) Forest Research Institute Baden-Wuerttemberg, Germany and at BOKU - University of Natural Ressources and Life Sciences, Vienna, Austria.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: grDevices, lubridate (>= 1.9), methods, rlang, stats, terra
        (>= 1.7), utils
Suggests: dplyr, ggplot2 (>= 3.4), knitr, rmarkdown, testthat (>=
        3.0.0), tidyterra (>= 0.4.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 17:40:28 UTC; lorenz
Author: Lorenz Bachfischer [aut, cre],
  Department of Forest Protection, FVA [fnd]
Maintainer: Lorenz Bachfischer <lorenz.bachfischer@posteo.de>
Repository: CRAN
Date/Publication: 2026-05-19 18:00:02 UTC
Built: R 4.5.2; ; 2026-05-19 21:39:36 UTC; unix

Package: PHENTHAUproc
Title: Phenology Modelling of Thaumetopoea Processionea
Version: 1.1.2
Authors@R: c(
    person("Lorenz", "Bachfischer", email = "lorenz.bachfischer@posteo.de", role = c("aut", "cre")),
    person("Department of Forest Protection, FVA", email = "Waldschutz.FVA-BW@forst.bwl.de", role = c("fnd")))
Description: Methods to calculate and present 'PHENTHAUproc', an early warning and decision support system for hazard assessment and control of oak processionary moth (OPM) using local and spatial temperature data. It was created by Halbig et al. 2024 (<doi:10.1016/j.foreco.2023.121525>) at FVA (<https://www.fva-bw.de/en/homepage/>) Forest Research Institute Baden-Wuerttemberg, Germany and at BOKU - University of Natural Ressources and Life Sciences, Vienna, Austria.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: grDevices, lubridate (>= 1.9), methods, rlang, stats, terra
        (>= 1.7), utils
Suggests: dplyr, ggplot2 (>= 3.4), knitr, rmarkdown, testthat (>=
        3.0.0), tidyterra (>= 0.4.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 17:40:28 UTC; lorenz
Author: Lorenz Bachfischer [aut, cre],
  Department of Forest Protection, FVA [fnd]
Maintainer: Lorenz Bachfischer <lorenz.bachfischer@posteo.de>
Repository: CRAN
Date/Publication: 2026-05-19 18:00:02 UTC
Built: R 4.6.0; ; 2026-05-19 22:21:26 UTC; unix

# Generated by roxygen2: do not edit by hand

export(convert_dwd_to_phenthau)
export(get_legend)
export(load_test)
export(mortality)
export(parameter)
export(phenology)
export(phenthau)
export(plot_date)
export(plot_stages)
export(plot_station_step)
importFrom(rlang,.data)
importFrom(stats,na.omit)

# PHENTHAUproc
## Phenology Modelling of Thaumetopoea Processionea

<!-- badges: start -->
<!-- badges: end -->

Methods to calculate and present 'PHENTHAUproc', an early warning and decision support system for hazard assessment and control of oak processionary moth (OPM) using local and spatial temperature data. It was created by Halbig et al. 2024 (<doi:10.1016/j.foreco.2023.121525>) at FVA (<https://www.fva-bw.de/en/homepage/>) Forest Research Institute Baden-Wuerttemberg, Germany and at BOKU - University of Natural Ressources and Life Sciences, Vienna, Austria.

## Installation

You can install the CRAN version directly or using the development version of PHENTHAUproc from [GitHub](https://github.com/) with:

``` r
#install.packages("PHENTHAUproc")

# install.packages("devtools")
devtools::install_github("bachfisch/PHENTHAUproc")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo = T, message = F, warning = F---------------------------------------
library(PHENTHAUproc)

# Packages used for demonstration:
library(terra)
library(tidyterra)
library(ggplot2)
library(grDevices)

## ----echo = T, message = F----------------------------------------------------
# local data for PHENTHAUproc 
freiburg <- load_test("hour")

head(freiburg)

## ----echo = T, message = F----------------------------------------------------
# spatial data for PHENTHAUproc
fva <-  load_test("SpatRaster")
fva

## ----echo = T, message = F----------------------------------------------------
# The parameter default method is "dailymeanminmax". If you want to calculate with hourly data, you have to change the parameter manually.
regional <- phenthau(fva)
local <- phenthau(freiburg, params = parameter("hour", year = 2020))

# we can create a parameter list
params <- parameter("dailymeanminmax", year = 2020)

# and change single parameter
params$budswelling$ldt <- 5 # change lower development threshold for budswelling from Default to 5

regional_manipulated <- phenthau(fva, params = params)

rm(params, regional_manipulated)

## ----echo = T, out.width = "100%", fig.dim = c(8, 6), message = T-------------
plot_station_step(local)

## ----echo = T, out.width = "50%", fig.align = "center", fig.cap = "OPM larval stages 5. June 2020", message = F----
stages <- regional$stages

plot_stages(stages,
            time = "2020-06-05",
            main = "OPM 5. June 2020",
            axes = F,
            box = T)

## ----echo = T, out.width = "50%", fig.cap = "Starvation related Mortality 2020" ,fig.align = "center", message = T----
# return mortality from list
mort <- regional$mortality

# get_legend returns a dataframe with ID, category and colors for a spatial PHENTHAUproc output
legend <- get_legend("mortality")

# set levels and colors for mortality
levels(mort) <- legend[,c("ID", "category")]
terra::coltab(mort) <- legend[,c("ID", "colors")]

# plot mortality FVA 2020
terra::plot(mort,
            plg = list(title = "%"),
            main = "Mortality 2020",
            all_levels = T,
            axes = F,
            box = T)

## ----echo = T, message = F, warning = F---------------------------------------
# show possible models and parametrisation: parameter()
leafunfolding <- phenology(fva,
                           model = "leafunfolding",
                           parametrisation = "quercus_robur_clone256_type1")


## ----echo = F, out.width = "100%", fig.dim = c(8, 6), fig.align = "center", message = F, warning = F----

mi <- min(terra::minmax(leafunfolding)["min",])
ma <- max(terra::minmax(leafunfolding)["max",])
me <- round(mean(c(mi, ma)))

ggplot() +
  geom_spatraster(data = leafunfolding) +
  scale_fill_whitebox_c(
    palette = "gn_yl",
    labels = lubridate::as_date,
    breaks = c(mi, me, ma),
    guide = guide_colorbar(
      title.position = "top",
      ticks.colour = "white",
      ticks.linewidth = 0.3
    )
  ) +
  theme_bw() +
  labs(title = "Leafunfolding 2020", subtitle = "4*4 pixel cutout centered at FVA", fill = "leafunfolding") +
  theme(legend.position = "right")

## ----echo = T, message = F, warning = F---------------------------------------
custers <- phenology(fva, model = "hatch", parametrisation = "custers")
meurisse <- phenology(fva, model = "hatch", parametrisation = "meurisse")
wagenhoff <- phenology(fva, model = "hatch", parametrisation = "wagenhoff")

## ----echo = F, out.width = "100%", fig.dim = c(8, 6), fig.align = "center", message = F, warning = F----
hatch <- c(custers, meurisse, wagenhoff)
names(hatch) <- c("custers", "meurisse", "wagenhoff")

mi <- min(terra::minmax(hatch)["min",])
ma <- max(terra::minmax(hatch)["max",])
me <- round(mean(c(mi, ma)))

ggplot() +
  geom_spatraster(data = hatch) +
  facet_wrap(~lyr, ncol = 3) +
  scale_fill_whitebox_c(
    palette = rev("viridi"),
    direction = -1,
    labels = lubridate::as_date,
    breaks = c(mi, me, ma),
    guide = guide_colorbar(
      direction = "horizontal",
      title.position = "top",
      barwidth = 20,
      draw.ulim = 1,
      draw.llim = 1
    )
  ) +
  theme_bw() +
  labs(title = "Hatchmodels 2020",
       subtitle = "4*4 pixel cutout centered at FVA",
       fill = "hatchday") +
  theme(legend.position = "bottom",
        axis.text.x = element_blank(),
        axis.text.y = element_blank())

#' Calculate first TRUE
#'
#' @param ... SpatRasters/SpatRaster list - logical - with time attribute
#' @returns SpatRaster - logical - with time attribute
#' @family Calculation
#' @keywords internal
#' @description Takes multiple logical SpatRasters with time attribute and
#' returns SpatRaster with common time and TRUE if any SpatRaster is TRUE.

calc_first <- function(...) {

  li <- list(...)

  if (methods::is(li[[1]], "list")) {

    if (length(li) == 1) li <- li[[1]] else stop("Cant calc_first of multiple lists!")

  }

  if (!methods::is(li[[1]], "SpatRaster")) stop("Input needs to be SpatRaster!")

  # take shared common time
  time <- lapply(li, terra::time)

  common <- Reduce(
    function(x, y) x[x %in% y],
    time
  )

  # subset all SpatRaster to common time
  sr <- lapply(li,
               function(x) terra::subset(x, which(terra::time(x) %in% common))
  )

  # reduce to one SpatRaster
  first <- Reduce(
    function(x, y) terra::ifel(x > y, x, y),
    sr
  )

  return(first)

}

#' Create parameter list
#'
#' @param model type of model for phenthau function - character - Either single
#' model or model collection
#' @param parametrisation type of parametrisation - character
#' @param year year of prognosis - numeric - Default: actual year
#' @param first logical - If TRUE and parametrisation is missing first
#' parametrisation in parameter() is used
#' @returns
#' If no argument is specified returns a list of parameter (used inside phenthau
#' function).
#' Otherwise returns available parameter for given model, parametrisation and
#' year
#' @family Main
#' @description
#'
#' See all available models with parameter: parameter()
#' model can be a single model or a model collection described in "Default
#' settings".
#' Return a data.frame with all model options:
#' parameter()
#'
#' \strong{Default Settings}
#'
#' The default parameter lists for different data input are:
#' "dailymean": Regional PHENTHAUproc described in Halbig et al. 2024 for daily
#' mean temperature data
#' "hour": "Local PHENTHAUproc described in Halbig et al. 2024 for daily hourly
#' temperature data
#' "dailymeanminmax": PHENTHAUproc adapted to DWD Data for daily mean, min and
#' max temperature data
#'
#' \strong{Columns}
#'
#' model: model
#' parametrisation: parametrisation
#' method: method used to calculate effective temperatures
#' ts_start: first day to calculate effective temperatures
#' ts_end last day to calculate effective temperatures (Default 30. Sept)
#' ts_prevyear: If True calculation of effective temperatures starts in previous
#' year. (i.e. wagenhoff)
#' ldt: lower development threshold
#' cf_dependent: Is model cold/frost dependent
#' cf_start: first day to calculate cold/frost days
#' cf_end: last day to calculate cold/frost days
#' cf_prevyear:  If True calculation of cold/frost days starts in previous year.
#' cf_temp: tmean" for cold days and "tmin" for frost days
#' cf_limit: threshold for cold/frost days
#' set: formula to calculate sum of effective temperatures
#' a: parameter for set
#' b: parameter for set
#'
#' @export
#' @examples
#'
#' # Default parameter list for daily mean, min and max temperature data:
#' parameter("dailymeanminmax")
#'
#' # overview dataframe with all available parameter sets
#' parameter("all")
#'
#' # all hatch model parameter
#' parameter("hatch")
#'
#' # return parameter necessary for calculation
#' parameter("hatch", "custers", 2020)

parameter <- function(model = NULL, parametrisation = NULL, year = NULL, first = TRUE) {

  # read parameter file
  p <- readRDS(system.file("config", "parameter.rds", package = "PHENTHAUproc", mustWork = TRUE))

  # show all parameter as a dataframe
  if (is.null(model)) return(p)
  if (model == "all") return(p)

  # set year if not given
  if (is.null(year)) {

    year <- as.numeric(format(Sys.Date(), "%Y"))

  }

  # give parameter for daily mean temperature data
  if (model == "dailymean") {

    p <- rbind(
      p[p$model == "budswelling" & p$parametrisation == "quercus_robur_clone256_type1",],
      p[p$model == "leafunfolding" & p$parametrisation == "quercus_robur_clone256_type1",],
      p[p$model == "hatch" & p$parametrisation == "meurisse" & p$method == "tsum",],
      p[p$model == "L2" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "L3" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "L4" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "L5" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "L6" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "Pp" & p$parametrisation == "degreedays" & p$method == "tsum",],
      p[p$model == "Ad" & p$parametrisation == "degreedays" & p$method == "tsum",]
    )

    # give parameter for hourly temperature data
  } else if (model == "hour") {

    p <- rbind(
      p[p$model == "budswelling" & p$parametrisation == "quercus_robur_clone256_type1",],
      p[p$model == "leafunfolding" & p$parametrisation == "quercus_robur_clone256_type1",],
      p[p$model == "hatch" & p$parametrisation == "custers",],
      p[p$model == "hatch" & p$parametrisation == "wagenhoff",],
      p[p$model == "hatch" & p$parametrisation == "meurisse",],
      p[p$model == "L2" & p$method == "tsum",],
      p[p$model == "L3" & p$method == "tsum",],
      p[p$model == "L4" & p$method == "tsum",],
      p[p$model == "L5" & p$method == "tsum",],
      p[p$model == "L6" & p$method == "tsum",],
      p[p$model == "Pp" & p$method == "tsum",],
      p[p$model == "Ad" & p$method == "tsum",]
    )

    # give parameter for daily mean min and max temperature data
  } else if (model == "dailymeanminmax") {

    p <- rbind(
      p[p$model == "budswelling" & p$parametrisation == "quercus_robur_clone256_type1" ,],
      p[p$model == "leafunfolding" & p$parametrisation == "quercus_robur_clone256_type1",],
      p[p$model == "hatch" & p$parametrisation == "custers",],
      p[p$model == "hatch" & p$parametrisation == "wagenhoff",],
      p[p$model == "hatch" & p$parametrisation == "meurisse",],
      p[p$model == "L2" & p$method == "baskerville",],
      p[p$model == "L3" & p$method == "baskerville",],
      p[p$model == "L4" & p$method == "baskerville",],
      p[p$model == "L5" & p$method == "baskerville",],
      p[p$model == "L6" & p$method == "baskerville",],
      p[p$model == "Pp" & p$method == "baskerville",],
      p[p$model == "Ad" & p$method == "baskerville",]
    )

  } else if (!model %in% p$model) {

    stop(model, " has to be one of model parameter list. see parameter(model = \"all\")")

  } else {

    p <- p[model == p$model,]

    if (is.null(parametrisation)) {

      if (first) p <- p[1,] else return(p)

    } else p <- p[parametrisation == p$parametrisation,]

  }

  if (nrow(p) == 0) stop("No model found for model = ", model, " and parametrisation = ", parametrisation, "!")

  # stupid hack because of hatchmodel appears multiple times
  listnames <- ifelse(p$model != "hatch", p$model, p$parametrisation)

  p <- split(p, listnames)

  p <- lapply(p, function(x) {

    x$ts_start <- get_date(year, x$ts_start, x$ts_prevyear)
    x$ts_end <- get_date(year, x$ts_end)

    if (x$cf_dependent) {

      x$cf_start <- get_date(year, x$cf_start, x$cf_prevyear)
      x$cf_end <- get_date(year, x$cf_end)

    } else {

      x$cf_start <- NULL
      x$cf_end <- NULL
      x$cf_temp <- NULL
      x$cf_limit <- NULL

    }

    x$ts_prevyear <- NULL
    x$cf_prevyear <- NULL

    x <- as.list(x)

    if (is.na(x$set)) x$set <- NA else x$set <- eval(parse(text = paste0(x$set)))

    return(x)

  }

  )

  if (model %in% c("dailymean", "dailymeanminmax", "hour")) {

    p$reftime <- model
    p$year <- year

  }

  return(p)

}

#' Calculate phenological models
#'
#' @param x SpatRaster list - with tmean, tmin, tmax and time attribute
#' @param params parameter list
#' @returns SpatRaster - logical - with event occurred/not TRUE/FALSE
#' @family Models
#' @keywords internal
#' @description This function is checking the dependencies of the model and
#' dependent on them:
#'
#' 1 Calculate the needed sum of effective (SET) temperatures for the
#' phenological event to happen
#' 2 Calculate the summed degree days (depending on hatch or not)
#' 3 Check if the summed degree days have reached the needed sum of effective
#' temperatures

calc_phenology <- function(x,
                           params) {

  ### 01 check parameter list for data input dependencies ----------------------

  check_data_with_params(x, params)

  # check dependencies
  cf_dependent <- params$cf_dependent
  hatch_dependent <- params$hatch_dependent

  ### 02 calculate sum of effective temperatures -------------------------------

  if (cf_dependent) {

    args <- get_formalArgs(params, calc_sumefftemp)
    args$x <- x
    sumefftemp <- do.call(calc_sumefftemp, args)
    sumefftemp <- subset_time(sumefftemp, params$ts_start, params$ts_end)
    rm(args)

  } else{

    sumefftemp <- params$a

  }

  ### 03 calculate degree days -------------------------------------------------

  args <- get_formalArgs(params, calc_degreedays)

  if (hatch_dependent) {

    args$x <- remove_false(x, params$hatch)

  } else args$x <- x

  degreedays <- do.call(calc_degreedays, args)

  ### 04 check if phenological event happened ----------------------------------

  event <- terra::ifel(degreedays > sumefftemp, TRUE, FALSE)

  # summarise hourly data if necessary
  event <- terra::tapp(event, index = "days", fun = "any")

  # create layernames
  names(event) <- timename(event, params$model)

  return(event)

}

#' Convert dataframe to SpatRaster list
#'
#' @param x dataframe (tmean, tmax, tmin, date)
#' @param date_col - character - name for date column which will be used to
#' create time attribute of SpatRaster.
#' @returns SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute
#' @family SpatRaster transformation
#' @keywords internal
#' @description
#' Internal function to create a SpatRaster list out of a dataframe.
#'
#' By default the dataframe should have a date column called "date".
#' The function is transferring each column into a list object and this object
#' to a 1 col, 1 row nrow nlyr SpatRaster.

convert_df_to_srl <- function(x, date_col = "date") {

  if (!is.data.frame(x)) stop("x has to be dataframe")

  names <- names(x)[names(x) != date_col]
  lyr <- nrow(x)
  date <- x[, date_col]

  # create list and transform columns to Spatraster
  li <- list()

  for (i in names) {
    if (!is.numeric(x[, i])) stop(i, " is not numeric!")
    li[[i]] <- x[, i]
    li[[i]] <- array(li[[i]], dim = c(1, 1, lyr))
    li[[i]] <- terra::rast(li[[i]])

    terra::time(li[[i]]) <- lubridate::ymd(date)

  }

  if (length(li) == 1) li <- li[[1]]

  return(li)

}

test_that("convert_hour_to_meanminmax",{
  x <- load_test("hour")[1:48, c("date", "tmean")]
  x <- convert_df_to_srl(x)
  x <- convert_hour_to_meanminmax(x)

  tmean <- round(as.data.frame(x[["tmean"]]), 1)
  tmin  <- round(as.data.frame(x[["tmin"]]),  1)
  tmax  <- round(as.data.frame(x[["tmax"]]),  1)

  expect_equal(tmean[[1]], 21.4)
  expect_equal(tmean[[2]], 17.5)
  expect_equal(tmin[[1]],  18.8)
  expect_equal(tmin[[2]],  11.6)
  expect_equal(tmax[[1]],  24)
  expect_equal(tmax[[2]],  22)
})

#' Calculate sum of effective temperatures (SET)
#'
#' @param x SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute
#' @param cf_temp - character - "tmean" for cold days and "tmin" for frost days
#' @param cf_start - Date - start of cold/frost timeseries
#' @param cf_end - Date - end of cold/frost timeseries
#' @param cf_limit - numeric - threshold under which a day is defined as
#' cold/frost day
#' @param set - function - x is cold/frost days, a and b are parameter i.e.
#' function (x, a, b) a + b*x
#' @param a - numeric - parameter for set function
#' @param b - numeric - parameter for set function
#' @returns SpatRaster with sum of effective temperatures.
#' @family Models
#' @keywords internal

calc_sumefftemp <- function(x,
                            cf_temp,
                            cf_start,
                            cf_end,
                            cf_limit,
                            set,
                            a,
                            b) {

  # subset to for cold/frost important days
  x <- subset_time(x[[cf_temp]], cf_start, cf_end)

  # calculate sum of effective temperatures using the formula set and the
  # cold_frost days
  # determine if day has a temperature under the threshold cf_limit
  x <- terra::ifel(x < cf_limit, 1, 0)
  x <- cumsum(x)

  # calculating sum of effective temperatures with cold_frost days and the set
  # function

  time <- terra::time(x)
  x <- terra::app(x, set, a = a, b = b)
  terra::time(x) <- time

  # rename with time
  names(x) <- timename(x, "sumefftemp")

  return(x)

}

#' Convert logical to day of year (doy)
#'
#' @param x SpatRaster - logical - with time attribute
#' @param from - Date - ymd - only if SpatRaster should be subset before being
#' transformed.
#' @param to - Date - ymd - only if SpatRaster should be subset before being
#' transformed.
#' @returns SpatRaster - numeric - day of year
#' @family SpatRaster transformation
#' @keywords internal
#' @description Converts logical SpatRaster with time attribute and one layer
#' per day to single layer with the day of the year (doy) where the layer is
#' 1/TRUE first.

convert_logical_to_doy <- function(x, from = NULL, to = NULL) {

  if (is.null(from)) from <- min(terra::time(x))
  if (is.null(to)) to <- max(terra::time(x))

  if (lubridate::year(from) < lubridate::year(to)) {

    from <- lubridate::ymd(paste0(lubridate::year(to), "01-01"))

  }

  name <- names(x)[terra::time(x) == to]

  time_vec <- seq(from, to, by = "days")
  x <- terra::subset(x, which(terra::time(x) %in% time_vec))
  doy <- lubridate::yday(terra::time(x[[1]]))
  out <- terra::ifel(x[[1]], doy, 0)

  if (terra::nlyr(x) > 1) {

    for (i in 2:terra::nlyr(x)) {
      doy <- lubridate::yday(terra::time(x[[i]]))
      out <- terra::ifel(out == 0 & x[[i]] != 0, doy, out)

    }
  }

  terra::time(out) <- to
  names(out) <- name

  return(out)
}

#' Load test data
#'
#' @param type character, either day/hour/SpatRaster/SpatRaster_hour/budswelling
#' @returns data.frame/SpatRaster list with test data
#' @family Helper
#' @export


load_test <- function(type = "SpatRaster") {

  if (type == "day") return(load_test_day())
  if (type == "hour") return(load_test_hour())
  if (type == "SpatRaster") return(load_test_srl())
  if (type == "SpatRaster_hour") return(load_test_srl_hour())
  if (type == "budswelling") return(load_test_bs())

  stop("testfile with name ", type, " not found.")

}


#' load dataframe - daily tmean tmin tmax - Freiburg 2020
#' @keywords internal

load_test_day <- function() {

  f <- system.file("extdata", "freiburg.csv", package = "PHENTHAUproc", mustWork = TRUE)

  x <- utils::read.csv(f)

  return(x)
}


#' load dataframe - hourly tmean - Freiburg 2023
#' @keywords internal

load_test_hour <- function() {

  f <- system.file("extdata", "freiburg_hour.csv", package = "PHENTHAUproc", mustWork = TRUE)

  x <- utils::read.csv(f)

  return(x)

}


#' load SpatRaster list - daily - 16 Rasterpoints around FVA 2020
#' @keywords internal

load_test_srl <- function() {

  tmean <- terra::rast(system.file("extdata", "fva_tmean.nc", package = "PHENTHAUproc", mustWork = TRUE))
  tmax <- terra::rast(system.file("extdata", "fva_tmax.nc", package = "PHENTHAUproc", mustWork = TRUE))
  tmin <- terra::rast(system.file("extdata", "fva_tmin.nc", package = "PHENTHAUproc", mustWork = TRUE))

  li <- list(tmean, tmax, tmin)
  names(li) <- c("tmean", "tmax", "tmin")

  return(li)
}


#' load SpatRaster - hourly - 16 Rasterpoints around FVA 2022 - 2023
#' @keywords internal

load_test_srl_hour <- function() {

  thour <- terra::rast(system.file("extdata", "fva_hour.nc", package = "PHENTHAUproc", mustWork = TRUE))

  return(thour)
}

#' load SpatRaster - budswelling - day of year - 16 Rasterpoints around FVA 2023
#' @keywords internal

load_test_bs <- function() {

  bs <- terra::rast(system.file("extdata", "fva_bs.nc", package = "PHENTHAUproc", mustWork = TRUE))

  return(bs)
}

test_that("phenthau", {

  x <- load_test("day")
  fr_day <- phenthau(x)

  expect_equal(fr_day$day[fr_day$model == "L2"], "2020-04-18")
  expect_equal(fr_day$day[fr_day$model == "L3"], "2020-04-27")
  expect_equal(fr_day$day[fr_day$model == "L4"], "2020-05-10")
  expect_equal(fr_day$day[fr_day$model == "L5"], "2020-05-25")
  expect_equal(fr_day$day[fr_day$model == "L6"], "2020-06-11")
  expect_equal(fr_day$day[fr_day$model == "Pp"], "2020-07-05")
  expect_equal(fr_day$day[fr_day$model == "Ad"], "2020-08-11") # should be 2020-08-12, but we can explain the difference with slightly different input than in Dr. Halbigs study

  expect_equal(fr_day$day[fr_day$model == "custers"], "2020-03-20")
  expect_equal(fr_day$day[fr_day$model == "wagenhoff"], "2020-03-18")
  expect_equal(fr_day$day[fr_day$model == "meurisse"], "2020-03-11")

})

#' Plot local PHENTHAUproc in a step plot
#'
#' @param x output of phenthau - dataframe - local PHENTHAUproc
#' @returns A plot showing local PHENTHAUproc results in a step plot.
#' @description Plots the development stages of OPM.
#' @importFrom rlang .data
#' @importFrom stats na.omit
#' @family Plot
#' @export
#' @examples
#'
#' fr_df <- load_test("day")
#' fr <- phenthau(fr_df)
#'
#' plot_station_step(fr)

plot_station_step <- function(x) {

  # required packages
  rlang::check_installed("ggplot2")

  # legend
  le <- get_legend("stages")
  le$id_col <- le$ID
  le$ID <- NULL

  # mortality
  # mortality <- as.numeric(x[x$model == "mortality", "day"])
  # x <- x[x$model != "mortality",]

  # limits and year
  x$day <- lubridate::ymd(x$day, quiet = TRUE)
  year <- unique(lubridate::year(na.omit(x$day)))
  from <- lubridate::ymd(paste0(year, "-03-01"))
  to <- lubridate::ymd(paste0(year, "-09-30"))
  last_day <- x[x$model == "last_day", "day"]

  # stages
  stages <- x[x$model %in% c("L1", "L2", "L3", "L4", "L5", "L6", "Pp", "Ad"), ]
  stages <- merge(le, stages, by.x = "category", by.y = "model", all.x = TRUE)
  stages <- stages[order(stages$id_col),]

  stages$xmin <- stages$day
  stages$xmin[1] <- from
  stages$xmax <- c(stages$xmin[-1], to)
  stages$ymin <- 0
  stages$ymax <- stages$id_col
  stages[!is.na(stages$xmin) & is.na(stages$xmax), "xmax"] <- last_day

  stages <- stages[!is.na(stages$xmin),]

  #ppa_biocide
  bio_start <- x[x$model == "ppa_biocide_start", "day"]
  bio_end <- x[x$model == "ppa_biocide_end", "day"]

  if (is.na(bio_end) & !is.na(bio_start)) bio_end <- last_day

  ## Plot
  # build boxes for stages
  g <- ggplot2::ggplot(data = stages) +

    ggplot2::geom_rect(
      ggplot2::aes(
        xmin = .data$xmin,
        xmax = .data$xmax,
        ymin = .data$ymin,
        ymax = .data$ymax),
      fill = stages$colors,
      alpha = 0.5
    ) +

    # build box for ppa_biocide
    ggplot2::annotate(
        geom = "rect",
        xmin = bio_start,
        xmax = bio_end,
        ymin = 0.25,
        ymax = 0.75,
        alpha = 0.5,
      color = "green",
      fill = "green"
    ) +

    # set x and y axis
    ggplot2::scale_y_continuous(
      breaks = 0:8,
      labels = c("Egg", "L1", "L2", "L3", "L4", "L5", "L6", "Pp", "Ad"),
      limits = c(0,8)
    ) +

    ggplot2::scale_x_date(
      date_breaks = "1 month",
      #date_labels = "%B",
      labels = \(x) month.name[lubridate::month(x)],
      limits = c(from, to)
    ) +

    # draw step line
    ggplot2::geom_step(
      ggplot2::aes(x = .data$xmin, y = .data$ymax),
      linewidth = 1,
      direction = "hv"
    ) +

    # set last segment of stepline
    ggplot2::geom_segment(
      data = stages[nrow(stages),],
      ggplot2::aes(
        x = .data$xmin,
        xend = .data$xmax,
        y = .data$id_col,
        yend = .data$id_col
      ),
      linewidth = 1
    ) +

    # text
    ggplot2::labs(
      x = "Month",
      y = "Stage",
      title = "Oak processionary moth",
      subtitle = paste0("Development - ", year)
    ) +

    # theme
    ggplot2::theme(
      legend.position = "none"
    ) +

    ggplot2::theme_bw()


  return(g)
}

#' Regional Weather data
#'
#' A dataset containing daily mean, minimum and maximum temperatures in °C.
#' The dataset is a 4*4 pixel cutout centered at FVA from the "Hyras" dataset
#' available at the DWD open data center
#' (https://opendata.dwd.de/climate_environment/CDC/grids_germany/daily/hyras_de/)
#'
#' The dataset can be loaded using load_test("SpatRaster").
#'
#' Spatial resolution: 5 km x 5 km
#' Projection: ETRS89 / LCC Europe (EPSG:3034)
#' Parameter: air temperature at 2 m
#'
#' The dataset is a list with three SpatRaster as objects.
#' The time attribute for all three SpatRaster is equal.
#'
#' The list objects are:
#'
#' \itemize{
#'   \item tmean: SpatRaster with tmean daily mean temperature in °C - numeric
#'   \item tmax: SpatRaster with tmax daily mean temperature in °C - numeric
#'   \item tmin: SpatRaster with tmin daily mean temperature in °C - numeric
#' }
#'
#' @docType data
#' @name regional
#' @keywords datasets
#' @format A list of SpatRaster
NULL

#' Get legend for PHENTHAUproc models
#'
#' @param x legend name - character - Available legends: "stages", "mortality",
#' "ppa_biocide"
#' @returns dataframe with ID, category and colors
#' @family Main
#' @export
#' @examples
#' # return legend for development stages
#' get_legend("stages")
#'

get_legend <- function(x = "stages") {

  av_leg <- c("stages", "mortality", "ppa_biocide")

  if (!x %in% av_leg) message("Available legends:\n", paste(av_leg, "\n"))

  if (x == "stages") {

    legend <- data.frame(
      "ID" = c(0:8),
      "category" = c("Egg", "L1", "L2", "L3", "L4", "L5", "L6", "Pp", "Ad"),
      "colors" = c(NA, "grey", "dimgrey", "lightyellow", "yellow", "orange", "red", "lightgrey", "darkgrey")
    )

  }

  if (x == "mortality") {

    legend <- data.frame(
      "ID" = c(0, 5, 10, 25, 50, 75, 100),
      "category" = as.character(c(0, 5, 10, 25, 50, 75, 100)),
      "colors" = rev(grDevices::hcl.colors(7, "Reds 2"))
    )

  }

  if (x == "ppa_biocide") {

    legend <- data.frame(
      "ID" = c(0, 1, 2),
      "category" = c("not yet effective", "possible", "not effective anymore"),
      "colors" = c("white", "forestgreen", "yellow")
    )

  }

  return(legend)

}

#' Calculate temperature sum
#'
#' @param tmean numeric - mean temperature
#' @param ldt numeric - lower development threshold aka base temperature.
#' @description
#' The function is vectorized.
#' To use it with SpatRaster-Data see ?terra::lapp.
#'
#' @returns The temperature sum over the lower development threshold.
#' @family Temperature sum methods
#' @keywords internal

calc_tsum <- function(tmean, ldt) {

  tmean[tmean <= ldt & !is.na(tmean)] <- 0
  tmean[tmean > ldt & !is.na(tmean)] <- tmean[tmean > ldt & !is.na(tmean)] - ldt

  return(tmean)
}

#' Local daily Weather Station data from Freiburg
#'
#' A dataset containing daily mean, max and min temperatures for Freiburg from
#' 2019-09-01 to 2020-09-30.
#' Downloaded from opendata.dwd.de and preprocessed.
#'
#' The dataset can be loaded using load_test("day").
#'
#' Stations_id: 01443
#' Stationsname: Freiburg
#'
#' The variables are as follows:
#'
#' \itemize{
#'   \item date in year-month-day - character
#'   \item tmean daily mean temperature in °C - numeric
#'   \item tmax daily max temperature in °C - numeric
#'   \item tmin daily min temperature in °C - numeric
#' }
#'
#' @docType data
#' @keywords datasets
#' @name local_daily
#' @source https://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/daily/kl/historical/
#' @format A data frame with 396 rows and 4 variables
NULL

#' Get Date format out of numeric year and monthday character value
#' @param year numeric year
#' @param monthday character with month and day i.e. ("-02-01" or "Feb 01")
#' @param prevyear if TRUE starts with previous year
#' @keywords internal
#' @family Helper

get_date <- function(year, monthday, prevyear = FALSE) {

  year <- ifelse(prevyear, year - 1, year)

  day <- lubridate::as_date(
    lubridate::ymd(
      paste0(year, monthday),
      tz = NULL),
    tz = NULL)

  return(day)

}

#' Convert day of year (doy) to logical
#'
#' @param x SpatRaster - numeric - value is day of year
#' @param from Date - ymd - first day of time attribute
#' @param to Date - ymd - last day of time attribute
#' @param by character - either "days", "weeks" or "months". Reduces return to
#' one day per day/week/month
#' @returns SpatRaster - logical - TRUE/FALSE before_doy/equal_after_doy
#' @family SpatRaster transformation
#' @keywords internal

convert_doy_to_logical <- function(x, from, to, by = "days") {

  time_vec <- seq(from, to, by = by)
  doy_vec <- lubridate::yday(time_vec)

  x_log <- lapply(doy_vec, function(y) {
    out <- terra::ifel(x > 0 & x <= y, 1, 0)
    terra::time(out) <- y
    return(out)
  })

  x_log <- terra::rast(x_log)
  terra::time(x_log) <- time_vec

  return(x_log)

}

#' Create a name with time stamp
#'
#' @param x SpatRaster with time attribute
#' @param model character - name of model
#' @param desc character - additional description (optional)
#' @family Helper
#' @keywords internal

timename <- function(x, model, desc = NULL) {

  time <- terra::time(x)

  if (!is.null(desc)) model <- paste(model, desc, "_")

  newname <- paste0(model, "_", gsub("-", "", paste(time)))

  return(newname)

}

#' Calculate PHENTHAUproc model
#'
#' @param x SpatRaster list/SpatRaster/data.frame - numeric - with time
#' attribute/date column (see Details for Input requirements)
#' @param params list with parameter (see Details for Input requirements)
#' @param def_hatch "first" or "mean": PHENTHAUproc has three hatch models
#' integrated (for details see: Custers 2003, Wagenhoff et al. 2014, Meurisse et
#' al. 2012). With "first" (Default) the first hatch-model which predicts hatch,
#' with "mean", the mean day of all hatch-models will be used for further
#' calculations.
#' @param budswelling SpatRaster/numeric with DOY (Day of year) - If budswelling
#' is provided, internal calculation of budswelling will be replaced. For raster
#' input provide a raster with same extend and crs as x.
#' @returns data.frame or list of SpatRaster with all PHENTHAUproc model outputs
#' (see Details)
#' @family Main
#' @description
#' "phenthau" implements the early warning system PHENTHAUproc created by
#' Halbig et al. 2024 in R.
#'
#' @details
#'
#' \strong{Overview}
#'
#' phenthau function combines multiple phenology models:
#' - bud swelling & leaf unfolding of \emph{Quercus robur} or
#' \emph{Quercus petraea}(still in development)
#' - hatch of oak processionary moth using 3 different hatch models
#' (Custers 2003, Wagenhoff et al. 2014 and Meurisse et al. 2012)
#' - development stages of OPM (0 egg - 8 adult)
#'
#' \strong{Input requirements}
#'
#' For different input type different parameter sets are needed. If params is
#' not provided it will be selected dependent on datatype and names(x) so follow
#' the name convention!
#'
#' daily raster input with tmean, tmin and tmax:
#' - a list of 3 named objects: "tmean", "tmin" and "tmax"
#' - each object is a SpatRaster with one layer per day
#' - terra::time for each SpatRaster has to be set to Date
#' -> parameter("dailymeanminmax", year)
#'
#' daily raster input with tmean:
#' - a SpatRaster
#' - one layer per day
#' - terra::time has to be set to Date
#' - params has to be set manually!
#' -> parameter("dailymean", year)
#'
#' hourly raster input:
#' - a SpatRaster
#' - one layer per hour
#' - terra::time has to be set to Date
#' -> parameter("hour", year)
#'
#' daily data.frame input with tmean, tmin and tmax:
#' - a data.frame with 4 columns: "date", "tmean", "tmin", "tmax"
#' - one row per day
#' -> parameter("dailymeanminmax", year)
#'
#' daily data.frame input with tmean:
#' - a data.frame with 2 columns: "date", "tmean"
#' - one row per day
#' - "date" column is.Date
#' -> parameter("dailymean", year)
#'
#' hourly data.frame input:
#' - a data.frame with 2 columns: "date", "hour", "tmean"
#' - 24 rows per day
#' - "date" column is.Date
#' -> parameter("hour", year)
#'
#' \strong{Parametrisation}
#'
#' Additional parametrization is provided but has not yet been tested.
#'
#' Use parameter() to return a data.frame with all possible parametrization
#' options or choose a model.
#' The default is "dailymeanminmax" and not dependent on the data input anymore.
#'
#' - "dailymean": Regional PHENTHAUproc how described in Halbig et al. 2024 for
#' daily mean temperature data
#' - "hour": Local PHENTHAUproc how described in Halbig et al. 2024 for hourly
#' temperature data
#' - "dailymeanminmax": PHENTHAUproc for daily mean, max and min temperature
#' data adapted to open access DWD raster data
#'
#' \strong{Output}
#'
#' Regional Output: \cr
#' A list with all model calculations as SpatRaster objects:
#'
#' - stages: SpatRaster with one layer per day - numeric - values from 0-8
#' (0 egg stage - 8 adult stage).
#'  -> use get_legend("stages") to show id/cover/colors
#'
#' - custers/wagenhoff/meurisse - one layer per day - logical - TRUE/FALSE for
#' hatch or no hatch
#' - budswelling: one layer per day - logical - TRUE/FALSE for budswelling or no
#' budswelling
#' - leafunfolding: one layer per day - logical - TRUE/FALSE for leafunfolding
#' or no leafunfolding
#'  -> plot the first day of a logical SpatRaster with plot_date()
#'
#' - mortality: one layer - integer - mortality in %
#' - ppa_biocide: one layer per day - numeric - 0 application of plant
#' protection agents (PPA) and biocides not yet effective, 1 application
#' effective, 2 appliaction not effective anymore
#'
#' Local Output: \cr
#' A data.frame with two columns:
#' - model: name of model
#' - date: date of first appearing of event
#'
#' \strong{Presentation}
#'
#' Regional Output:
#' - plot_stages is a wrapper around terra::plot to preset legend, names, colors
#' and day to plot.
#' - plot_date is a wrapper around terra::plot to plot the date of first TRUE in
#' multiple layered SpatRaster
#'
#' Local Output:
#' - plot_station_step is for local weather station data and creates a stepwise
#' graph for the development stages
#'
#' @references
#'
#' Halbig et al. 2024: Halbig, P., Stelzer, A. S., Baier, P., Pennerstorfer, J.,
#' Delb, H., & Schopf, A. (2024). PHENTHAUproc–An early warning and decision
#' support system for hazard assessment and control of oak processionary moth
#' (Thaumetopoea processionea). Forest Ecology and Management, 552, 121525 \cr
#' Custers 2003: Custers, C. (2003). Climate change and trophic synchronisation.
#' English Wageningen UR, Chairgroup Environmental Systems Analysis.\cr
#' Wagenhoff et al. 2014: Wagenhoff, E., Wagenhoff, A., Blum, R., Veit, H.,
#' Zapf, D., & Delb, H. (2014). Does the prediction of the time of egg hatch of
#' Thaumetopoea processionea (Lepidoptera: Notodontidae) using a frost
#' day/temperature sum model provide evidence of an increasing temporal mismatch
#' between the time of egg hatch and that of budburst of Quercus robur due to
#' recent global warming?. European Journal of Entomology, 111(2).\cr
#' Meurisse et al. 2012: Meurisse, N., Hoch, G., Schopf, A., Battisti, A.,
#' & Grégoire, J. C. (2012). Low temperature tolerance and starvation ability
#' of the oak processionary moth: implications in a context of increasing
#' epidemics. Agricultural and forest entomology, 14(3), 239-250.\cr
#'
#' @author Bachfischer Lorenz, Department of Forest Protection FVA (2024)
#'   \email{lorenz.bachfischer@@posteo.de}
#' @export
#' @examples
#'
#' srl <- load_test()
#' phen <- phenthau(srl)

phenthau <- function(x,
                     params = NULL,
                     def_hatch = "first",
                     budswelling = NULL
) {

  ### 01 set parameter ---------------------------------------------------------

  year <- lubridate::year(max(get_time(x)))


  ### 02 check and transform input ---------------------------------------------

  if (methods::is(x, "data.frame")) {

    is_dataframe <- TRUE

    # for dailymean structure
    if (identical(names(x), c("date", "tmean"))) {

      if (is.null(params)) params <- parameter("dailymean", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean")])

    # for hourly structure
    } else if (identical(names(x), c("date", "hour", "tmean"))) {

      if (is.null(params)) params <- parameter("hour", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean")])

      # for dailymeanminmax structure
    } else if (all(names(x) %in% c("date", "tmean", "tmin", "tmax"))) {

      if (is.null(params)) params <- parameter("dailymeanminmax", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean", "tmin", "tmax")])

    }

  } else if (methods::is(x, "list")) {

    is_dataframe <- FALSE

    lapply(
      x,
      function(sr) {
        if (!methods::is(sr, "SpatRaster")) stop("If calculating with list of SpatRaster all list objects have to be SpatRaster.")
      }
    )

    if (all(names(x) %in% c("tmean", "tmin", "tmax"))) {

      params <- parameter("dailymeanminmax", year = year)

    } else stop("Name convention not correct. SpatRaster list needs three objects. These objects must be tmean, tmin and tmax!")


  } else if (methods::is(x, "SpatRaster")) {

    is_dataframe <- FALSE

    params <- parameter("hour", year = year)

  } else {

    stop("x has to be a SpatRaster for hourly raster data, a list of SpatRaster with tmean, tmin and tmax for daily raster data or a data.frame for daily or hourly weather station data.")

  }

  reftime <- params$reftime

  # Convert hourly to daily mean, min and max temperatures (needed for most of
  # the models)
  if (reftime == "hour") {

    x_hour <- list("tmean" = x)
    x <- convert_hour_to_meanminmax(x_hour$tmean)

  }


  ### 03 set and check times ---------------------------------------------------

  # write the times with lubridate
  x <- lapply(x, function(y) {
    terra::time(y) <- lubridate::ymd(terra::time(y), tz = NULL)
    return(y)
  })

  # check if days complete
  time <- get_time(x)
  year <- lubridate::year(max(time))
  from <- lubridate::as_date(paste0(year - 1, "-09-01"), tz = NULL)
  to <- lubridate::as_date(paste0(year, "-09-30"), tz = NULL)

  if (to > max(time)) to <- max(time)

  message("Calculating PHENTHAUproc:")
  message("Temperature data input: ", reftime)
  message("Output datatype: ", if (is_dataframe) "dataframe" else "list of SpatRaster")
  message("Year of Prognosis: ", year)
  message("First day of calculation: ", from)
  message("Last day of calculation: ", to)

  sequence <- seq(from, to, by = "days")

  missing <- sum(!sequence %in% time)

  if (missing > 0) warning(missing, " missing days in time sequence.")


  ### 04 Calculate Quercus phenology -------------------------------------------

  # use budswelling if provided

  if (!is.null(budswelling)) {

    message("Budswelling is provided. Internal calculation is not used!")

    if (methods::is(budswelling, "SpatRaster")) {

      if (terra::crs(budswelling) != terra::crs(x[[1]])) {

        stop("Budswelling needs to have the same crs and extend as input raster.")

      }

      budswelling <- convert_doy_to_logical(budswelling,
                                            from = params$budswelling$ts_start,
                                            to = to)

    } else if (methods::is(budswelling, "numeric")) {

      budswelling <- array(budswelling, dim = c(1, 1, 1))
      budswelling <- terra::rast(budswelling)

      budswelling <- convert_doy_to_logical(budswelling,
                                            from = params$budswelling$ts_start,
                                            to = to)

    } else stop("budswelling needs to be a SpatRaster or a numeric value")

  } else budswelling <- calc_phenology(x, params$budswelling)

  # calculate leafunfolding
  leafunfolding <- calc_phenology(x, params$leafunfolding)


  ### 05 Calculate hatch -------------------------------------------------------

  par_hatch <- c("custers", "wagenhoff", "meurisse")[
    c("custers", "wagenhoff", "meurisse") %in% names(params)
  ]

  hatch <- list()
  hatch[par_hatch] <- lapply(params[par_hatch], function(y) calc_phenology(x, y))

  hatch <- subset_time(hatch, from = params$L2$ts_start, to = params$L2$ts_end)

  # If first, the model which calculates the earliest hatch is taken
  # If mean the mean day of the different models is taken

  # calculate hatch
  if (def_hatch == "first") L1 <- calc_first(hatch)
  if (def_hatch == "mean") L1 <- calc_mean(hatch)


  ### 06 L1 feeding start and ppa/biocide application --------------------------

  # feeding starts with budswelling if hatch is not later
  L1_feeding_start <- terra::ifel(budswelling & L1, 1, 0)

  larvae <- c("L2", "L3", "L4", "L5", "L6", "Pp", "Ad")

  # add data to parameter lists
  # The development of the larvae starts with L1_feeding_start.
  params[larvae] <- lapply(params[larvae], function(y) {
    y$hatch <- L1_feeding_start
    return(y)
  })


  ### 07 calculate larval stages -----------------------------------------------

  # calculate hatch-dependent models
  stages <- list("L1" = L1)

  temp_larvae <- if (reftime == "hour") x_hour else x

  stages[larvae] <- lapply(params[larvae], function(y) calc_phenology(temp_larvae, y))


  ### 08 mortality & PPA/biocide application -----------------------------------

  mortality <- calc_mortality(x, budswelling, L1)
  names(mortality) <- timename(mortality, "mortality")

  ppa_biocide <- terra::ifel(leafunfolding, 1, 0)
  ppa_biocide <- terra::ifel(stages$L4, 2, ppa_biocide)
  names(ppa_biocide) <- timename(ppa_biocide, "ppa_biocide")


  ### 09 Output ----------------------------------------------------------------

  # for dataframes
  if (is_dataframe) {

    out <- c(
      hatch,
      "budswelling" = budswelling,
      "leafunfolding" = leafunfolding,
      "L1_feeding_start" = L1_feeding_start,
      "ppa_biocide_start" = NA,
      "ppa_biocide_end" = NA,
      stages)

    last_day <- max(get_time(stages))

    out$ppa_biocide_start <- ppa_biocide == 1
    out$ppa_biocide_end <- ppa_biocide == 2

    out <- lapply(out, convert_logical_to_time)

    out <- c(out, "mortality" = mortality)

    out <- lapply(out, function(x) as.numeric(terra::values(x)))

    out <- data.frame("model" = names(out),
                      "day" = as.character(out),
                      row.names = NULL)

    out[out$model != "mortality", "day"] <-
      as.character(
        lubridate::as_date(as.numeric(out[out$model != "mortality", "day"]))
      )

    out <- rbind(out, data.frame(model = "last_day", day = as.character(last_day)))

  } else {

    # for SpatRaster

    stages <- Reduce(f = "+", stages)

    names(stages) <- timename(stages, "phenthau")

    out <- c(hatch,
             "budswelling" = budswelling,
             "leafunfolding" = leafunfolding,
             "ppa_biocide" = ppa_biocide,
             "mortality" = mortality,
             "stages" = stages)

  }

  return(out)
}

#' Check data for model requirements
#'
#' @param x SpatRaster list - numeric - with time attribute
#' @param params parameter list
#' @returns No return value, called for side effects.
#' @family Helper
#' @keywords internal

check_data_with_params <- function(x, params) {

  time <- get_time(x)

  if (!params$ts_start %in% time) stop("ts_start not in time attribute/vector!")
  if (params$cf_dependent && !params$cf_start %in% time) stop("cf_start not in time attribute/vector!")
  if (params$hatch_dependent && is.null(params$hatch)) stop("model is hatch dependent so hatch has to be provided in data")

}

#' Calculate degree days using the single sine method of Baskerville
#'
#' @param tmin min temperature - numeric
#' @param tmax max temperature - numeric
#' @param ldt lower development threshold aka base temperature - numeric
#' @returns Degree days calculated by using the single sine method of Baskerville.
#' @description
#'
#' The function is vectorized.
#' To use it with SpatRaster data see ?terra::lapp
#'
#' More Information about the Method itself:
#' Baskerville, G.L. and P. Emin. 1969. Rapid estimation of heat accumulation
#' from maximum and minimum temperature. Ecology 50:514-517.
#' @family Temperature sum methods
#' @keywords internal

calc_baskerville <- function(tmin, tmax, ldt) {

  tmean <- (tmin + tmax) / 2

  tmax[tmax <= ldt & !is.na(tmax)] <- 0

  tmax[tmin >= ldt & !is.na(tmin)] <- tmean[tmin >= ldt & !is.na(tmin)] - ldt

  sel <- tmin < ldt & tmax > ldt & !is.na(tmin) & !is.na(tmax)

  tmax[sel] <- (((((tmax[sel] - tmin[sel]) / 2) * cos(asin(((ldt - tmean[sel]) / ((tmax[sel] - tmin[sel]) / 2))))) +
                   ((tmean[sel] - ldt) * ((pi / 2) - asin(((ldt - tmean[sel]) / ((tmax[sel] - tmin[sel]) / 2)))))) / pi)

  tmax <- round(tmax, digits = 2)

  return(tmax)
}

#' Local hourly Weather Station data from Freiburg
#'
#' A dataset containing hourly temperatures for Freiburg from 2019-09-01 to
#' 2022-12-31.
#' Downloaded from opendata.dwd.de and preprocessed.
#'
#' The dataset can be loaded using load_test("hour").
#'
#' Stations_id: 01443
#' Stationsname: Freiburg
#'
#' The variables are as follows:
#'
#' \itemize{
#'   \item date in year-month-day - character
#'   \item tmean hourly mean temperature in °C - numeric
#' }
#'
#' @docType data
#' @keywords datasets
#' @name local_hourly
#' @source https://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/hourly/air_temperature/historical/
#' @format A data frame with 29232 rows and 3 variables
NULL

#' Convert hourly to daily
#'
#' @param x SpatRaster - numeric - hourly tmean with time attribute
#' @returns SpatRaster list (tmean, tmax, tmin) - numeric  - with time attribute
#' @family SpatRaster transformation
#' @keywords internal

convert_hour_to_meanminmax <- function(x){

  # check missing values

  time <- terra::time(x)
  count <- rle(sort(as.vector(time)))[["lengths"]]
  missing <- sum(24 - count)

  if (missing != 0) warning(missing, " missing hours in sequence.")

  # summarise hourly temperature to daily tmin, tmean, tmax

  tmean <- terra::tapp(x, index = "days", fun = "mean")
  names(tmean) <- timename(tmean, "tmean")

  tmin <- terra::tapp(x, index = "days", fun = "min")
  names(tmin) <- timename(tmin, "tmin")


  tmax <- terra::tapp(x, index = "days", fun = "max")
  names(tmax) <- timename(tmax, "tmax")

  # create output

  srl <- list("tmean" = tmean,
              "tmin" = tmin,
              "tmax" = tmax)

  return(srl)

}

#' Calculate phenological events
#'
#' @param x SpatRaster list/dataframe (tmean, tmax, tmin) - numeric - with time
#' attribute/date column
#' @param model name of model - character
#' @param parametrisation name of parametrisation - character
#' @param year year for prognosis - numeric
#' @param hatch SpatRaster - logical - with time attribute TRUE/FALSE
#' hatch/no_hatch
#' @param return_date TRUE/FALSE defines output -> see value
#' @param ... parameter to change default values. (i.e. ldt = 3.5)
#' @returns If return_date is TRUE returns single layered SpatRaster with time
#' serial number (first occurence of phenological event).
#' If return_date is FALSE returns a one layer per day SpatRaster type logical
#' with phenological event occurred/not TRUE/FALSE.
#' @family Main
#' @description Using daily mean or min and max temperature data, the function
#' calculates the temperature-dependent development stages of OPM or the bud
#' stages (bud swelling and leaf unfolding) of its host tree
#' \emph{Quercus robur}.
#'
#' The default settings correspond to the model described by Halbig et al. 2024.
#' Additional parametrizations are provided but have not yet been tested.
#'
#'  Halbig et al. 2024
#'   It follows 4 different steps:
#'   - a) Calculating and summing up cold days or frost days.
#'   (Cold days are defined as days with a mean temperature below ldt
#'   (lower development threshold), while frost days are all days with a min
#'   temperature below ldt).
#'      Hatch dependent development stages need a hatch raster (hatch happened
#'      1 or not 0) for each day
#'   - b) Calculating degree days with the
#'   single sine method of Baskerville & Emin, 1969 or simple summing up tmean
#'   temperatures over ldt.
#'   - c) Calculating the needed
#'   sum of effective temperatures for the development stage
#'   - d) Comparing degree days with the needed sum of effective temperatures
#'
#' @references
#'
#' Halbig et al. 2014: Halbig, P., Stelzer, A. S., Baier, P., Pennerstorfer,
#' J., Delb, H., & Schopf, A. (2024). PHENTHAUproc–An early warning and decision
#' support system for hazard assessment and control of oak processionary moth
#' (Thaumetopoea processionea). Forest Ecology and Management, 552, 121525 \cr
#' Baskerville & Emin 1969: Baskerville, G. L., & Emin, P. (1969). Rapid
#' estimation of heat accumulation from maximum and minimum temperatures.
#' Ecology, 50(3), 514-517. (<doi:10.2307/1933912>) \cr
#' Menzel 1997: Menzel, A. (1997). Phänologie von Waldbäumen unter sich
#' ändernden Klimabedingungen: Auswertung der Beobachtungen in den
#' internationalen phänologischen Gärten und Möglichkeiten der Modellierung von
#' Phänodaten. Frank. \cr
#'
#' @author Bachfischer Lorenz, Department of Forest Protection FVA (2024)
#'   \email{lorenz.bachfischer@@posteo.de}
#' @export
#' @examples
#' ## SpatRaster
#' srl <- load_test()
#'
#' # Calculating bud swelling for our raster example
#' budswelling <- phenology(srl,
#'                          model = "budswelling",
#'                          parametrisation = "quercus_robur_clone256_type1",
#'                          year = 2020)
#'


phenology <- function(x,
                      model,
                      parametrisation = NULL,
                      year = NULL,
                      hatch = NULL,
                      return_date = TRUE,
                      ...
                      ) {

  ### 01 set parameter ---------------------------------------------------------

  year <- lubridate::year(max(get_time(x)))


  ### 02 check and transform input ---------------------------------------------

  if (methods::is(x, "data.frame")) {

    is_dataframe <- TRUE

    # for dailymean structure
    if (identical(names(x), c("date", "tmean"))) {

      if (is.null(params)) params <- parameter("dailymean", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean")])

      # for hourly structure
    } else if (identical(names(x), c("date", "hour", "tmean"))) {

      if (is.null(params)) params <- parameter("hour", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean")])

      # for dailymeanminmax structure
    } else if (all(names(x) %in% c("date", "tmean", "tmin", "tmax"))) {

      if (is.null(params)) params <- parameter("dailymeanminmax", year = year)

      x <- convert_df_to_srl(x[,c("date", "tmean", "tmin", "tmax")])

    }

  } else if (methods::is(x, "list")) {

    is_dataframe <- FALSE

    lapply(
      x,
      function(sr) {
        if (!methods::is(sr, "SpatRaster")) stop("If calculating with list of SpatRaster all list objects have to be SpatRaster.")
      }
    )

    if (all(names(x) %in% c("tmean", "tmin", "tmax"))) {

      params <- parameter("dailymeanminmax", year = year)

    } else stop("Name convention not correct. SpatRaster list needs three objects. These objects must be tmean, tmin and tmax!")


  } else if (methods::is(x, "SpatRaster")) {

    is_dataframe <- FALSE

    params <- parameter("hour", year = year)

  } else {

    stop("x has to be a SpatRaster for hourly raster data, a list of SpatRaster with tmean, tmin and tmax for daily raster data or a data.frame for daily or hourly weather station data.")

  }

  reftime <- params$reftime

  # Convert hourly to daily mean, min and max temperatures (needed for most of
  # the models)
  if (reftime == "hour") {

    x_hour <- list("tmean" = x)
    x <- convert_hour_to_meanminmax(x_hour$tmean)

  }


  ### 03 set and check times ---------------------------------------------------

  # write the times with lubridate
  x <- lapply(x, function(y) {
    terra::time(y) <- lubridate::ymd(terra::time(y), tz = NULL)
    return(y)
  })

  # check if days complete
  time <- get_time(x)
  year <- lubridate::year(max(time))
  from <- lubridate::as_date(paste0(year - 1, "-09-01"), tz = NULL)
  to <- lubridate::as_date(paste0(year, "-09-30"), tz = NULL)

  if (to > max(time)) to <- max(time)

  sequence <- seq(from, to, by = "days")

  missing <- sum(!sequence %in% time)

  if (missing > 0) warning(missing, " missing days in time sequence.")


  ### 03 create parameter list -------------------------------------------------

  params <- parameter(model = model,
                      parametrisation = parametrisation,
                      year = year)[[1]]

  if (!is.null(hatch)) params$hatch <- hatch

  my_params <- list(...)
  params <- replace(params, names(my_params), my_params)


  ### 04 calculate phenology ---------------------------------------------------

  out <- calc_phenology(x, params)

  names(out) <- timename(out, model)

  ### 05 create output ---------------------------------------------------------

  if (return_date) out <- convert_logical_to_time(out)

  if (is_dataframe) {

    out <- convert_sr_to_cvec(out)
    names(out) <- model

  }

  return(out)
}

#' Call a function from PHENTHAUproc package
#'
#' @param f function
#' @param data SpatRaster list (tmean, tmax, tmin) - numeric - with time
#' attribute
#' @param params parameter list
#' @family Helper
#' @keywords internal
#' @description calls a function from PHENTHAUproc package

call_function <- function(f, data, params) {

  fA <- methods::formalArgs(f)

  if ("data" %in% fA) params$data <- data else params <- append(data, params)

  if (is.list(params)) params <- params[which(names(params) %in% fA)]

  if (is.null(params)) warning("Parameterlist for function ", f, "is empty.")

  return(do.call(f, params))

}

#' Calculate degree days
#'
#' @param x SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute
#' @param ts_start start of timeseries - Date
#' @param ts_end end of timeseries - Date
#' @param ldt lower development threshold - numeric
#' @param method name of degreedays/temperature sum method - character - either
#' "baskerville" or "tsum"
#' @family Models
#' @keywords internal
#' @returns SpatRaster of growing (summed up) degree days.

calc_degreedays <- function(x,
                            ts_start,
                            ts_end,
                            ldt,
                            method) {

  x <- subset_time(x, ts_start, ts_end)

  # check method
  if (!method %in% c("baskerville", "tsum")) stop("method has to be one of temperature sum methods: ", list("\nbaskerville ", "\ntsum"))
  if (method == "baskerville") f <- calc_baskerville
  if (method == "tsum") f <- calc_tsum

  # select necessary temperatures
  minmeanmax <- c("tmin", "tmean", "tmax")
  temperature <- minmeanmax[minmeanmax %in% names(formals(f))]

  x <- x[temperature]
  time <- get_time(x)
  x <- terra::sds(x)
  degree_days <- terra::lapp(x, f, usenames = T, ldt = ldt)
  terra::time(degree_days) <- time

  degree_days <- cumsum(degree_days)

  names(degree_days) <- timename(degree_days, "degreedays")

  # we need the summed up degree days for comparision with sum of effective
  # temperatures

  return(degree_days)

}

#' Calculate last TRUE
#'
#' @param x SpatRaster - logical - with time attribute
#' @returns SpatRaster with time serial number (origin = lubridate::origin) of
#' last TRUE layer.
#' @family Calculation
#' @keywords internal
#' @description Takes multiple logical SpatRasters with time attribute and
#' returns SpatRaster with time serial number of last TRUE layer time.

calc_last <- function(x) {

  time <- terra::time(x)
  layer <- 1:terra::nlyr(x)
  ma <- matrix(c(rev(layer), time), ncol = 2, byrow = F)

  x <- terra::which.lyr(x[[rev(layer)]])
  #x <- terra::as.factor(x)
  x <- terra::classify(x, rcl = ma)
  names(x) <- "timeserialnumber"

  return(x)

}

#' Calculate mean TRUE
#'
#' @param ... SpatRasters/SpatRaster list - logical - with time attribute
#' @returns SpatRaster - logical - with time attribute
#' @family Calculation
#' @keywords internal
#' @description Takes multiple logical SpatRasters with time attribute and
#' returns SpatRaster with the mean time serial number of first and last TRUE
#' value.

calc_mean <- function(...) {

  li <- list(...)

  if (methods::is(li[[1]], "list")) {

    if (length(li) == 1) li <- li[[1]] else stop("Cant calc_first of multiple lists!")

  }

  if (!methods::is(li[[1]], "SpatRaster")) stop("Input needs to be SpatRaster!")

  # take shared common time
  time <- lapply(li, terra::time)

  common <- Reduce(
    function(x, y) x[x %in% y],
    time
  )

  # subset all SpatRaster to common time
  sr <- lapply(li,
               function(x) terra::subset(x, which(terra::time(x) %in% common))
  )

  ishatch <- Reduce(function(x, y) sum(x, y), sr)

  # reduce to one SpatRaster
  meanhatch <- lapply(sr, terra::which.lyr)
  meanhatch <- terra::rast(meanhatch)
  meanhatch <- terra::mean(meanhatch, na.rm = TRUE)

  out <- sr[[1]]
  time <- terra::time(out)

  for (i in 1:terra::nlyr(out)) {

    out[[i]] <- terra::ifel(ishatch[[i]] > 1, i >= meanhatch, ishatch[[i]])

  }

  terra::time(out) <- time

  return(out)

}

#' Convert logical to time
#'
#' @param x SpatRaster - logical - with time attribute
#' @returns SpatRaster - numeric
#' @family SpatRaster transformation
#' @keywords internal
#' @description Converts logical SpatRasters to single layer SpatRaster with timeserialnumber as value for first TRUE

convert_logical_to_time <- function(x) {

    time <- terra::time(x)
    layer <- 1:terra::nlyr(x)
    ma <- matrix(c(layer, time), ncol = 2, byrow = F)

    x <- terra::which.lyr(x)
    #x <- terra::as.factor(x)
    x <- terra::classify(x, rcl = ma)
    names(x) <- "timeserialnumber"

    return(x)


}

#' Subset data.frame/SpatRaster/List within from and end time
#'
#' @param x data.frame with time column, SpatRaster with time attribute or List
#' of SpatRasters with same time attributes
#' @param from first day of subset
#' @param to last day of subset
#' @family Helper
#' @keywords internal


subset_time <- function(x, from, to) {

  from <- lubridate::ymd(from)
  to <- lubridate::ymd(to)

  # create sequence
  sequence <- seq(from, to, by = "days")

  # choose subset depending on type
  if (methods::is(x, "data.frame")) x <- x[lubridate::ymd(x$time) %in% sequence, ]

  # create spatraster subset function
  f <- function(y)  terra::subset(y, which(terra::time(y) %in% sequence))

  if (methods::is(x, "SpatRaster")) x <- f(x)

  if (methods::is(x, "list")) x <- lapply(x, f)

  if (methods::is(x, "SpatRasterDataset")) {

    names <- names(x)

    x <- lapply(x, f)
    x <- terra::sds(x)

    names(x) <- names

  }

  return(x)
}

#' Set attributes for SpatRaster
#'
#' @param x SpatRaster - to assign attributes to
#' @param type character - type of attributes i.e. "stages"
#' @family Helper
#' @keywords internal


set_attributes <- function(x, type) {

  lev <- get_legend(type)[,c("ID", "category")]

  for (i in 1:terra::nlyr(x)) levels(x[[i]]) <- lev

  #names(x) <- timename(x, type)

  return(x)

}

#' Plot regional PHENTHAUproc - Stages
#'
#' @param x SpatRaster stages output of phenthau - numeric
#' @param time day to plot - Date or character year-month_day i.e.("2020-05-01")
#' @param ... arguments passed along to terra::plot
#' @returns A plot of the PHENTHAUproc stages of the last/chosen time with
#' preset levels and colors.
#' @family Plot
#' @description
#' A wrapper to plot the development stages of OPM with assigned names and
#' colors
#'
#' @details phenthau returns a list of SpatRasters. The stages object describes
#' the development stages of oak processionary moth.
#' This function plots the stages SpatRaster with the right names and colors.
#' To get IDs, caregories and colors use get_legend("stages").
#'
#' @export

plot_stages <- function(x,
                        time = NULL,
                        ...) {

  if (is.null(time)) time <- max(terra::time(x))
  if (terra::nlyr(x) > 1) x <- terra::subset(x, which(terra::time(x) == time))

  leg <- get_legend("stages")

  levels(x) <- leg[,c("ID", "category")]
  terra::coltab(x) <- leg[,c("ID", "colors")]

  terra::plot(x,
              all_levels = TRUE,
              ...)

}

#' Convert SpatRaster to characer vector
#'
#' @param x SpatRaster - numeric - value is timeserialnumber
#' @returns vector - Date as character
#' @family SpatRaster transformation
#' @keywords internal
#' @description
#' Transform local PHENTHAUproc results back from SpatRaster to vector format

convert_sr_to_cvec <- function(x) {

  x <- terra::values(x)
  x <- as.numeric(x)
  x <- lubridate::as_date(x)
  x <- as.character(x)

  return(x)

}

#' Remove FALSE
#'
#' @param x SpatRaster/SpatRaster list - numeric - with time attribute
#' @param y SpatRaster - logical - with time attribute
#' @family Helper
#' @keywords internal
#' @description
#' Set values in SpatRaster or SpatRaster list to -999 if y is FALSE

remove_false <- function(x, y) {

  # remove x which is not covered by y
  ty <- get_time(y)
  start <- min(ty)
  end <- max(ty)

  x <- subset_time(x, start, end)

  # choose daily layers multiple times when x is hourly

  names(y) <- as.character(ty)
  tx <- as.character(get_time(x))
  y <- y[[tx]]

  # overwrite y is FALSE with -999

  x <- lapply(x, function(z) terra::ifel(y, z, -999))

  return(x)

}

#' Get time attribute
#'
#' @param x SpatRaster, data.frame, list of SpatRaster or SpatRasterDataset
#' @returns If x is a data.frame returns the time column, else it returns the
#' time attribute of a SpatRaster or the shared time attribute of multiple
#' SpatRaster.
#' @family Helper
#' @keywords internal

get_time <- function(x) {

  if (methods::is(x, "SpatRaster")) time <- terra::time(x)

  if (methods::is(x, "data.frame")) time <- lubridate::ymd(x$date, tz = "")

  if (methods::is(x, "list") | methods::is(x, "SpatRasterDataset")) {

    time <- lapply(x, terra::time)
    f <- function(x, y){return(x[x %in% y])}
    time <- Reduce(f, time)

  }

  return(time)
}

#' Calculating starvation related mortality of \emph{Thaumetopoea processionea}
#'
#' @param x SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute
#' @param par_budswelling character - parametrisation for bud swelling
#' @param par_hatch character - parametrisation for hatch
#' @param def_hatch definition of hatch - character - either "first" or "mean"
#' @param last If TRUE returns only the result for the last day. last row/layer
#' for data.frame/SpatRaster
#' @returns SpatRaster with mortality in %.
#' @description Calculating the starvation-related mortality rate of L1 larvae,
#' dependent on degree days from the first hatch to feeding start (bud swelling
#' of the host tree \emph{Quercus robur}).
#' All already available parameter settings can be shown with parameter().
#' @family Main
#' @export
#' @examples
#' \donttest{
#' srl <- load_test()
#' mortality(srl)
#'}

mortality <- function(x,
                      par_budswelling = "quercus_robur_clone256_type1",
                      par_hatch = c("custers", "wagenhoff", "meurisse"),
                      def_hatch = "first",
                      last = TRUE) {

  ### 1 check and convert input -------------


  is_hour <- NULL

  if (all(c("date", "hour", "tmean") %in% names(x))) {
    is_hour <- TRUE
  } else if (all(c("tmean", "tmin", "tmax") %in% names(x))) {
    is_hour <- FALSE
  } else {
    stop("names(x) not like needed. See details.")
  }

  # convert data.frame into list of SpatRaster (srl)
  if (methods::is(x, "data.frame")) {

    x <- convert_df_to_srl(x[, names(x) != "hour"])

    is_dafr <- TRUE # if input is a dataframe, output should be too.

  } else is_dafr <- FALSE

  # transfer hourly to daily minmeanmax (needed for most of the models)
  if (is_hour) {

    x <- convert_hour_to_meanminmax(x)

  }

  # write the times with lubridate
  x <- lapply(x, function(y) {
    terra::time(y) <- lubridate::ymd(get_time(y))
    return(y)
  })

  # check if days complete
  time <- get_time(x)
  year <- lubridate::year(max(time))
  from <- lubridate::as_date(paste0(year - 1, "-09-01"), tz = NULL)
  to <- lubridate::as_date(paste0(year, "-09-30"), tz = NULL)

  if (to > max(time)) to <- max(time)

  sequence <- seq(from, to, by = "days")

  missing <- sum(!sequence %in% time)

  if (missing > 0) warning(missing, " missing days in sequence.")

  ### 02 Calculate Phenology models ------------------------------

  budswelling <- phenology(x,
                           model = "budswelling",
                           parametrisation = par_budswelling,
                           year = year,
                           return_date = FALSE
                           )

  hatchmodels <- lapply(par_hatch, function(y) phenology(x, "hatch", y, year, return_date = FALSE))

  # calculate hatch
  if (def_hatch == "first") hatch <- calc_first(hatchmodels)
  if (def_hatch == "mean") hatch <- calc_mean(hatchmodels)

  mortality <- calc_mortality(x, budswelling, hatch)

  if (last) mortality <- mortality[[terra::nlyr(mortality)]]

  if (is_dafr) {

    mortality <- as.numeric(terra::values(mortality))
    names(mortality) <- "mortality"

  }

  return(mortality)
}

#' Get formalArgs of a function from a list of parameter
#'
#' @param p parameter list
#' @param f function
#' @family Helper
#' @keywords internal

get_formalArgs <- function(p, f){

  fA <- names(formals(f))
  p <- p[names(p) %in% fA]

  return(p)

}

#' Check dimension and time attribute of a SpatRaster list
#'
#' @param ... SpatRaster/SpatRaster list/data.frame/vector
#' @returns No return value, called for side effects.
#' @family Helper
#' @keywords internal

check_dimension_and_time <- function(...) {

  li <- list(...)

  if (methods::is(li[[1]], "list")) {
    if (length(li) != 1) warning("Multiple list objects as input. Only first will be used!")

    li <- li[[1]]
  }

  if (methods::is(li[[1]], "SpatRaster")) {
    share_geom <- length(unique(lapply(li, terra::crs))) == 1

    if (!share_geom) stop("SpatRaster dont share same geometry!")

    time_vec <- lapply(li, terra::time)

    share_time <- length(unique(time_vec)) == 1

    if (!share_time) stop("SpatRaster dont share same time vector!")
  }

  if (methods::is(li[[1]], "vector")) {
    share_length <- length(unique(lapply(li, nrow))) == 1

    if (!share_length) stop("Vectors dont have the same length!")
  }

}

#' Convert hourly DWD temperature data to PHENTHAUproc input
#'
#' @param x filepath to unzipped DWD temperature data (text file)
#' @returns A dataframe with date, hour and mean air temperature (tmean).
#' @family Helper
#' @export

convert_dwd_to_phenthau <- function(x) {

  # read DWD raw data
  x <- utils::read.csv2(x)

  # extract date, hour and tmean
  x$date <- substr(x$MESS_DATUM, 1, 8)
  x$hour <- substr(x$MESS_DATUM, 9, 10)
  x$tmean <- as.numeric(x$TT_TU)

  # set names to input names necessary for phenthau function
  x <- x[ ,c("date", "hour", "tmean")]

  x <- x[order(x$date, x$hour), ]

  return(x)

}

#' Calculate mortality
#'
#' @param x SpatRaster list / dataframe with temperature data and time
#' attribute/column
#' @param budswelling SpatRaster - logical - T/F budswelling/no_budswelling
#' @param hatch SpatRaster - logical - T/F hatch/no_hatch
#' @param mot mortality threshold - numeric - is used as lower development
#' threshold calculating degree days
#' @returns SpatRaster of starving related mortality in %.
#' @family Models
#' @keywords internal

calc_mortality <- function(x, budswelling, hatch, mot = 0) {

  check_dimension_and_time(budswelling, hatch)

  starve <- terra::ifel(hatch & !budswelling, TRUE, FALSE)

  from <- min(get_time(starve))
  to <- max(get_time(starve))

  x <- subset_time(x, from, to)
  time <- terra::time(x$tmean)

  x <- lapply(x, function(y) terra::ifel(starve, y, -999))

  x <- lapply(x, function(y) {
    terra::time(y) <- time
  return(y)
  }
  )

  dd <- calc_degreedays(x, ts_start = from, ts_end = to, ldt = mot, method = "tsum")

  mortality_tab <- matrix(
    c(
      0, 130, 0,
      130, 147, 5,
      147, 187, 10,
      187, 210, 25,
      210, 238, 50,
      238, 313, 75,
      313, 9999, 100
    ),
    ncol = 3,
    byrow = T
  )

  mortality <- terra::classify(dd, mortality_tab)
  mortality <- mortality[[terra::nlyr(mortality)]]

  return(mortality)
}

#' Plot SpatRaster with date serial number/time attribute
#'
#' @param x SpatRaster - numeric - value is serial number or SpatRaster -
#' logical - with time attribute
#' @param breaks number of breaks in the legend - numeric
#' @param ... arguments for terra::plot function, except (type, breaks, col or
#' plg)
#' @returns A plot of a phenological event by day with legend.
#' @description A wrapper around terra::plot to show the time serial number as
#' a character date in the legend.
#' @family Plot
#' @export
#' @examples
#'
#' data <- load_test()
#' budswelling <- phenology(data, "budswelling", "quercus_robur_clone256_type1", 2020)
#' plot_date(budswelling)

plot_date <- function(x,
                      breaks = NULL,
                      ...) {

  if (terra::nlyr(x) > 1) x <- convert_logical_to_time(x)

  first <- terra::minmax(x)["min", ]
  last <- terra::minmax(x)["max", ]

  if (is.null(breaks)) breaks <- last - first
  while (breaks > 20) breaks <- round(breaks / 2)

  days <- round(seq(first, last, length.out = breaks))
  days <- lubridate::as_date(days)


  terra::plot(x,
    type = "interval",
    breaks = breaks,
    plg = list(legend = days),
    ...
  )
}

---
title: "PHENTHAUproc"
author: "Bachfischer, Lorenz"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{PHENTHAUproc}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

PHENTHAUproc is an early warning system, modelling the life cycle of oak processionary moth (OPM, *Thaumetopoea processionea*) with temperature data. It was created by Halbig et al. 2024[^1] at [FVA – Forest Research Institute Baden-Wuerttemberg](https://www.fva-bw.de/en/), Germany and Institute Baden-Wuerttemberg, Germany and [BOKU](https://boku.ac.at/en/) - University of Natural Resources and Life Sciences, Vienna, Austria.

The aim is to support practitioners in risk assessment and planning of measures. It consists of temperature-sum based models to predict the phenological development of OPM stages, bud swelling and leaf unfolding of oak (*Quercus robur*), the starvation-dependent mortality of OPM L1 larvae as well as the application period of possible plant protection agents (ppa) and biocides.

This package implements the phenological models of PHENTHAUproc in R for local weather station data and spatial data, using the [terra](https://CRAN.R-project.org/package=terra/) package created by Robert J. Hijmans.

The models were developed, adjusted and tested using temperature data and OPM observation data from Brandenburg and Baden-Wuerttemberg, Germany. Consider this in calculating OPM development in other regions.


## Installation & setup

PHENTHAUproc is available on CRAN.

```{r, echo = T, message = F, warning = F}
library(PHENTHAUproc)

# Packages used for demonstration:
library(terra)
library(tidyterra)
library(ggplot2)
library(grDevices)
```

## Required data structure

PHENTHAUproc works with temperature-sum based models. The calculation requires daily mean, min and max air temperature in Celsius. For local prognosis, a data frame with the columns date, hour and tmean is required. For regional calculations, a named list of SpatRasters with equal time attribute is needed. The names of the list objects have to be tmean, tmin and tmax. The last available year will be used as year of prediction. Be aware that some models require temperature data from the previous year (Default: 1. Sept from previous year).

Weather station data and raster data for Germany can be found at [Climate Data Center](https://opendata.dwd.de/) of DWD (German Meteorological Service), the basis of our sample record. In our regional example, we use a 4*4 pixel cutout centered at FVA from the spatial data of the DWD dataset ["HYRAS"](https://opendata.dwd.de/climate_environment/CDC/grids_germany/daily/hyras_de/air_temperature_mean/).

Spatial resolution: 5 km x 5 km   
Projection: ETRS89 / LCC Europe (EPSG:3034)   
Parameter: mean min and max air temperature at 2 m    
time: from 2019-09-01 to 2020-09-30   

For local data, we use hourly temperature data in Celsius from the weather station of Freiburg im Breisgau(Stations_id: 01443) from 2019 to 2022. (You can use ?load_test() to load example datasets.)

```{r, echo = T, message = F}
# local data for PHENTHAUproc 
freiburg <- load_test("hour")

head(freiburg)
```


```{r, echo = T, message = F}
# spatial data for PHENTHAUproc
fva <-  load_test("SpatRaster")
fva
```

## PHENTHAUproc

### Calculation

With the phenthau function, all model predictions are calculated. For a data frame input, it will return a data frame with the computed start dates of the phenological events (like bud swelling, different larval stages, etc.) or % for OPM L1 larval mortality. For a SpatRasterlist input, phenthau returns a SpatRasterlist. The output will always be limited from 1. February to 30. September. For more output information see: ?phenthau

```{r, echo = T, message = F}
# The parameter default method is "dailymeanminmax". If you want to calculate with hourly data, you have to change the parameter manually.
regional <- phenthau(fva)
local <- phenthau(freiburg, params = parameter("hour", year = 2020))

# we can create a parameter list
params <- parameter("dailymeanminmax", year = 2020)

# and change single parameter
params$budswelling$ldt <- 5 # change lower development threshold for budswelling from Default to 5

regional_manipulated <- phenthau(fva, params = params)

rm(params, regional_manipulated)
```

### Results

#### Oak phenology
For oak phenology, bud swelling and leaf unfolding are calculated based on the work of Menzel 1997[^2]. Bud swelling determines the feeding start of hatched OPM L1 larvae. Leaf unfolding sets the start of the possible ppa/biocide usage. Leaf unfolding can be calculated for Quercus robur with different parametrisations related to the oak clones from which phenology observation data for model creation were obtained.

#### L1 - hatch
Hatch of OPM defines the beginning of the first larval stage L1. PHENTHAUproc includes three hatch models.

- Custers 2003[^3]  
- Meurisse et al. 2012[^4]  
- Wagenhoff et al. 2014[^5]

#### OPM - stages
Hatch and bud swelling are starting the feeding period of OPM and the calculation of the further development stages:  

- L2-L6 larval stages  
- Pp Pupa  
- Ad Adult  

#### Ppa/biocide application period
The application period of appropriate ppa/biocides starts with leaf unfolding of oak and ends with the beginning of larval stage (L4) of OPM.

#### Mortality
The starvation-related mortality is calculated. It depends on the temperature sum during the period from OPM L1 hatching to oak bud swelling. Lacking coincidence of these two phenological events can lead to mortality of the young larvae (Wagenhoff et al. 2013[^6], Halbig et al. 2024).


## Presentation of local PHENTHAUproc

One graph (currently only in English language) is available for local results and requires ggplot2. 

```{r, echo = T, out.width = "100%", fig.dim = c(8, 6), message = T}
plot_station_step(local)
```


## Presentation of regional PHENTHAUproc

### Stages

Stages is a SpatRaster with one layer per day and values from 0 to 8 for each development stage of OPM rom the egg stage to the adult stage (see phenthau_legend() for assignment). The plot_stages function is a wrapper around terra::plot to preset categories and colors and subset the required date (Default: max(date)).

```{r, echo = T, out.width = "50%", fig.align = "center", fig.cap = "OPM larval stages 5. June 2020", message = F}
stages <- regional$stages

plot_stages(stages,
            time = "2020-06-05",
            main = "OPM 5. June 2020",
            axes = F,
            box = T)
```

### Mortality

To assign colors and categories directly to the SpatRaster or to use it elsewhere, you can use the get_legend function.

```{r, echo = T, out.width = "50%", fig.cap = "Starvation related Mortality 2020" ,fig.align = "center", message = T}
# return mortality from list
mort <- regional$mortality

# get_legend returns a dataframe with ID, category and colors for a spatial PHENTHAUproc output
legend <- get_legend("mortality")

# set levels and colors for mortality
levels(mort) <- legend[,c("ID", "category")]
terra::coltab(mort) <- legend[,c("ID", "colors")]

# plot mortality FVA 2020
terra::plot(mort,
            plg = list(title = "%"),
            main = "Mortality 2020",
            all_levels = T,
            axes = F,
            box = T)
```

## Examples

### Example: Phenology of Quercus robur at Freiburg (FVA) in 2020

Single phenological models can be calculated with the phenology function using different parametrisations (see parameter() for all preset parameters). Single parameter can directly be manipulated to compare phenological models.

```{r, echo = T, message = F, warning = F}
# show possible models and parametrisation: parameter()
leafunfolding <- phenology(fva,
                           model = "leafunfolding",
                           parametrisation = "quercus_robur_clone256_type1")

```


```{r, echo = F, out.width = "100%", fig.dim = c(8, 6), fig.align = "center", message = F, warning = F}

mi <- min(terra::minmax(leafunfolding)["min",])
ma <- max(terra::minmax(leafunfolding)["max",])
me <- round(mean(c(mi, ma)))

ggplot() +
  geom_spatraster(data = leafunfolding) +
  scale_fill_whitebox_c(
    palette = "gn_yl",
    labels = lubridate::as_date,
    breaks = c(mi, me, ma),
    guide = guide_colorbar(
      title.position = "top",
      ticks.colour = "white",
      ticks.linewidth = 0.3
    )
  ) +
  theme_bw() +
  labs(title = "Leafunfolding 2020", subtitle = "4*4 pixel cutout centered at FVA", fill = "leafunfolding") +
  theme(legend.position = "right")
```

### Example: Hatch at Freiburg (FVA) in 2020

```{r, echo = T, message = F, warning = F}
custers <- phenology(fva, model = "hatch", parametrisation = "custers")
meurisse <- phenology(fva, model = "hatch", parametrisation = "meurisse")
wagenhoff <- phenology(fva, model = "hatch", parametrisation = "wagenhoff")
```

```{r, echo = F, out.width = "100%", fig.dim = c(8, 6), fig.align = "center", message = F, warning = F}
hatch <- c(custers, meurisse, wagenhoff)
names(hatch) <- c("custers", "meurisse", "wagenhoff")

mi <- min(terra::minmax(hatch)["min",])
ma <- max(terra::minmax(hatch)["max",])
me <- round(mean(c(mi, ma)))

ggplot() +
  geom_spatraster(data = hatch) +
  facet_wrap(~lyr, ncol = 3) +
  scale_fill_whitebox_c(
    palette = rev("viridi"),
    direction = -1,
    labels = lubridate::as_date,
    breaks = c(mi, me, ma),
    guide = guide_colorbar(
      direction = "horizontal",
      title.position = "top",
      barwidth = 20,
      draw.ulim = 1,
      draw.llim = 1
    )
  ) +
  theme_bw() +
  labs(title = "Hatchmodels 2020",
       subtitle = "4*4 pixel cutout centered at FVA",
       fill = "hatchday") +
  theme(legend.position = "bottom",
        axis.text.x = element_blank(),
        axis.text.y = element_blank())

```

### Example: Implementation with [leaflet](https://leafletjs.com/)

![Implementation with leaflet](bw_example.png)



[^1]: Halbig, P., Stelzer, A., Baier, P., Pennerstorfer, J., Delb, H., Schopf, A. 2023 PHENTHAUproc – An early warning and decision support system for hazard assessment and control of oak processionary moth (Thaumetopoea processionea) [doi](doi:10.1016/j.foreco.2023.121525)   
[^2]: Menzel, A. 1997 Phänologie von Waldbäumen unter sich ändernden Klimabedingungen - Auswertung der Beobachtungen in den Internationalen Phänologischen Gärten und Möglichkeiten der Modellierung von Phänodaten. Forstliche Forschungsberichte München, 164, 1-147.  
[^3]: Custers, C.J.L. 2003 Climate change and trophic synchronisation - A casestudy of the Oak Processionary Caterpillar. Master's Thesis, Wageningen University.  
[^4]: Meurisse, N., Hoch, G., Schopf, A., Battisti, A. and Grégoire, J.C. 2012 Low temperature tolerance and starvation ability of the oak processionary moth: implications in a context of increasing epidemics. Agricultural and Forest Entomology, 14, 239-250. [doi](doi:10.1111/j.1461-9563.2011.00562.x)  
[^5]: Wagenhoff, E., Wagenhoff, A., Blum, R., Veit, H., Zapf, D. and Delb, H. 2014 Does the prediction of the time of egg hatch of Thaumetopoea processionea (Lepidoptera: Notodontidae) using a frost day/temperature sum model provide evidence of an increasing temporal mismatch between the time of egg hatch and that of budburst of Quercus robur due to recent global warming? European Journal of Entomology, 111, 207-215. [doi](doi:10.14411/eje.2014.030)   
[^6]: Wagenhoff, E., Blum, R., Engel, K., Veit, H., Delb, H., 2013. Temporal synchrony of Thaumetopoea processionea egg hatch and Quercus robur budburst. J. Pest. Sci. 86, 193–202. [doi](<doi:10.1007/s10340-012-0457-7>)

\documentclass{article}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenthau.R
\name{phenthau}
\alias{phenthau}
\title{Calculate PHENTHAUproc model}
\usage{
phenthau(x, params = NULL, def_hatch = "first", budswelling = NULL)
}
\arguments{
\item{x}{SpatRaster list/SpatRaster/data.frame - numeric - with time
attribute/date column (see Details for Input requirements)}

\item{params}{list with parameter (see Details for Input requirements)}

\item{def_hatch}{"first" or "mean": PHENTHAUproc has three hatch models
integrated (for details see: Custers 2003, Wagenhoff et al. 2014, Meurisse et
al. 2012). With "first" (Default) the first hatch-model which predicts hatch,
with "mean", the mean day of all hatch-models will be used for further
calculations.}

\item{budswelling}{SpatRaster/numeric with DOY (Day of year) - If budswelling
is provided, internal calculation of budswelling will be replaced. For raster
input provide a raster with same extend and crs as x.}
}
\value{
data.frame or list of SpatRaster with all PHENTHAUproc model outputs
(see Details)
}
\description{
"phenthau" implements the early warning system PHENTHAUproc created by
Halbig et al. 2024 in R.
}
\details{
\strong{Overview}

phenthau function combines multiple phenology models:
\itemize{
\item bud swelling & leaf unfolding of \emph{Quercus robur} or
\emph{Quercus petraea}(still in development)
\item hatch of oak processionary moth using 3 different hatch models
(Custers 2003, Wagenhoff et al. 2014 and Meurisse et al. 2012)
\item development stages of OPM (0 egg - 8 adult)
}

\strong{Input requirements}

For different input type different parameter sets are needed. If params is
not provided it will be selected dependent on datatype and names(x) so follow
the name convention!

daily raster input with tmean, tmin and tmax:
\itemize{
\item a list of 3 named objects: "tmean", "tmin" and "tmax"
\item each object is a SpatRaster with one layer per day
\item terra::time for each SpatRaster has to be set to Date
-> parameter("dailymeanminmax", year)
}

daily raster input with tmean:
\itemize{
\item a SpatRaster
\item one layer per day
\item terra::time has to be set to Date
\item params has to be set manually!
-> parameter("dailymean", year)
}

hourly raster input:
\itemize{
\item a SpatRaster
\item one layer per hour
\item terra::time has to be set to Date
-> parameter("hour", year)
}

daily data.frame input with tmean, tmin and tmax:
\itemize{
\item a data.frame with 4 columns: "date", "tmean", "tmin", "tmax"
\item one row per day
-> parameter("dailymeanminmax", year)
}

daily data.frame input with tmean:
\itemize{
\item a data.frame with 2 columns: "date", "tmean"
\item one row per day
\item "date" column is.Date
-> parameter("dailymean", year)
}

hourly data.frame input:
\itemize{
\item a data.frame with 2 columns: "date", "hour", "tmean"
\item 24 rows per day
\item "date" column is.Date
-> parameter("hour", year)
}

\strong{Parametrisation}

Additional parametrization is provided but has not yet been tested.

Use parameter() to return a data.frame with all possible parametrization
options or choose a model.
The default is "dailymeanminmax" and not dependent on the data input anymore.
\itemize{
\item "dailymean": Regional PHENTHAUproc how described in Halbig et al. 2024 for
daily mean temperature data
\item "hour": Local PHENTHAUproc how described in Halbig et al. 2024 for hourly
temperature data
\item "dailymeanminmax": PHENTHAUproc for daily mean, max and min temperature
data adapted to open access DWD raster data
}

\strong{Output}

Regional Output: \cr
A list with all model calculations as SpatRaster objects:
\itemize{
\item stages: SpatRaster with one layer per day - numeric - values from 0-8
(0 egg stage - 8 adult stage).
-> use get_legend("stages") to show id/cover/colors
\item custers/wagenhoff/meurisse - one layer per day - logical - TRUE/FALSE for
hatch or no hatch
\item budswelling: one layer per day - logical - TRUE/FALSE for budswelling or no
budswelling
\item leafunfolding: one layer per day - logical - TRUE/FALSE for leafunfolding
or no leafunfolding
-> plot the first day of a logical SpatRaster with plot_date()
\item mortality: one layer - integer - mortality in \%
\item ppa_biocide: one layer per day - numeric - 0 application of plant
protection agents (PPA) and biocides not yet effective, 1 application
effective, 2 appliaction not effective anymore
}

Local Output: \cr
A data.frame with two columns:
\itemize{
\item model: name of model
\item date: date of first appearing of event
}

\strong{Presentation}

Regional Output:
\itemize{
\item plot_stages is a wrapper around terra::plot to preset legend, names, colors
and day to plot.
\item plot_date is a wrapper around terra::plot to plot the date of first TRUE in
multiple layered SpatRaster
}

Local Output:
\itemize{
\item plot_station_step is for local weather station data and creates a stepwise
graph for the development stages
}
}
\examples{

srl <- load_test()
phen <- phenthau(srl)
}
\references{
Halbig et al. 2024: Halbig, P., Stelzer, A. S., Baier, P., Pennerstorfer, J.,
Delb, H., & Schopf, A. (2024). PHENTHAUproc–An early warning and decision
support system for hazard assessment and control of oak processionary moth
(Thaumetopoea processionea). Forest Ecology and Management, 552, 121525 \cr
Custers 2003: Custers, C. (2003). Climate change and trophic synchronisation.
English Wageningen UR, Chairgroup Environmental Systems Analysis.\cr
Wagenhoff et al. 2014: Wagenhoff, E., Wagenhoff, A., Blum, R., Veit, H.,
Zapf, D., & Delb, H. (2014). Does the prediction of the time of egg hatch of
Thaumetopoea processionea (Lepidoptera: Notodontidae) using a frost
day/temperature sum model provide evidence of an increasing temporal mismatch
between the time of egg hatch and that of budburst of Quercus robur due to
recent global warming?. European Journal of Entomology, 111(2).\cr
Meurisse et al. 2012: Meurisse, N., Hoch, G., Schopf, A., Battisti, A.,
& Grégoire, J. C. (2012). Low temperature tolerance and starvation ability
of the oak processionary moth: implications in a context of increasing
epidemics. Agricultural and forest entomology, 14(3), 239-250.\cr
}
\seealso{
Other Main:
\code{\link[=get_legend]{get_legend()}},
\code{\link[=mortality]{mortality()}},
\code{\link[=parameter]{parameter()}},
\code{\link[=phenology]{phenology()}}
}
\author{
Bachfischer Lorenz, Department of Forest Protection FVA (2024)
\email{lorenz.bachfischer@posteo.de}
}
\concept{Main}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset_time.R
\name{subset_time}
\alias{subset_time}
\title{Subset data.frame/SpatRaster/List within from and end time}
\usage{
subset_time(x, from, to)
}
\arguments{
\item{x}{data.frame with time column, SpatRaster with time attribute or List
of SpatRasters with same time attributes}

\item{from}{first day of subset}

\item{to}{last day of subset}
}
\description{
Subset data.frame/SpatRaster/List within from and end time
}
\seealso{
Other Helper:
\code{\link[=call_function]{call_function()}},
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=convert_dwd_to_phenthau]{convert_dwd_to_phenthau()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=get_time]{get_time()}},
\code{\link[=load_test]{load_test()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=timename]{timename()}}
}
\concept{Helper}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_legend.R
\name{get_legend}
\alias{get_legend}
\title{Get legend for PHENTHAUproc models}
\usage{
get_legend(x = "stages")
}
\arguments{
\item{x}{legend name - character - Available legends: "stages", "mortality",
"ppa_biocide"}
}
\value{
dataframe with ID, category and colors
}
\description{
Get legend for PHENTHAUproc models
}
\examples{
# return legend for development stages
get_legend("stages")

}
\seealso{
Other Main:
\code{\link[=mortality]{mortality()}},
\code{\link[=parameter]{parameter()}},
\code{\link[=phenology]{phenology()}},
\code{\link[=phenthau]{phenthau()}}
}
\concept{Main}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_local_hourly.R
\docType{data}
\name{local_hourly}
\alias{local_hourly}
\title{Local hourly Weather Station data from Freiburg}
\format{
A data frame with 29232 rows and 3 variables
}
\source{
https://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/hourly/air_temperature/historical/
}
\description{
A dataset containing hourly temperatures for Freiburg from 2019-09-01 to
2022-12-31.
Downloaded from opendata.dwd.de and preprocessed.
}
\details{
The dataset can be loaded using load_test("hour").

Stations_id: 01443
Stationsname: Freiburg

The variables are as follows:

\itemize{
\item date in year-month-day - character
\item tmean hourly mean temperature in °C - numeric
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_dwd_to_phenthau.R
\name{convert_dwd_to_phenthau}
\alias{convert_dwd_to_phenthau}
\title{Convert hourly DWD temperature data to PHENTHAUproc input}
\usage{
convert_dwd_to_phenthau(x)
}
\arguments{
\item{x}{filepath to unzipped DWD temperature data (text file)}
}
\value{
A dataframe with date, hour and mean air temperature (tmean).
}
\description{
Convert hourly DWD temperature data to PHENTHAUproc input
}
\seealso{
Other Helper:
\code{\link[=call_function]{call_function()}},
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=get_time]{get_time()}},
\code{\link[=load_test]{load_test()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=subset_time]{subset_time()}},
\code{\link[=timename]{timename()}}
}
\concept{Helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_test.R
\name{load_test_srl}
\alias{load_test_srl}
\title{load SpatRaster list - daily - 16 Rasterpoints around FVA 2020}
\usage{
load_test_srl()
}
\description{
load SpatRaster list - daily - 16 Rasterpoints around FVA 2020
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_regional.R
\docType{data}
\name{regional}
\alias{regional}
\title{Regional Weather data}
\format{
A list of SpatRaster
}
\description{
A dataset containing daily mean, minimum and maximum temperatures in °C.
The dataset is a 4*4 pixel cutout centered at FVA from the "Hyras" dataset
available at the DWD open data center
(https://opendata.dwd.de/climate_environment/CDC/grids_germany/daily/hyras_de/)
}
\details{
The dataset can be loaded using load_test("SpatRaster").

Spatial resolution: 5 km x 5 km
Projection: ETRS89 / LCC Europe (EPSG:3034)
Parameter: air temperature at 2 m

The dataset is a list with three SpatRaster as objects.
The time attribute for all three SpatRaster is equal.

The list objects are:

\itemize{
\item tmean: SpatRaster with tmean daily mean temperature in °C - numeric
\item tmax: SpatRaster with tmax daily mean temperature in °C - numeric
\item tmin: SpatRaster with tmin daily mean temperature in °C - numeric
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_last.R
\name{calc_last}
\alias{calc_last}
\title{Calculate last TRUE}
\usage{
calc_last(x)
}
\arguments{
\item{x}{SpatRaster - logical - with time attribute}
}
\value{
SpatRaster with time serial number (origin = lubridate::origin) of
last TRUE layer.
}
\description{
Takes multiple logical SpatRasters with time attribute and
returns SpatRaster with time serial number of last TRUE layer time.
}
\seealso{
Other Calculation:
\code{\link[=calc_first]{calc_first()}},
\code{\link[=calc_mean]{calc_mean()}}
}
\concept{Calculation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_phenology.R
\name{calc_phenology}
\alias{calc_phenology}
\title{Calculate phenological models}
\usage{
calc_phenology(x, params)
}
\arguments{
\item{x}{SpatRaster list - with tmean, tmin, tmax and time attribute}

\item{params}{parameter list}
}
\value{
SpatRaster - logical - with event occurred/not TRUE/FALSE
}
\description{
This function is checking the dependencies of the model and
dependent on them:

1 Calculate the needed sum of effective (SET) temperatures for the
phenological event to happen
2 Calculate the summed degree days (depending on hatch or not)
3 Check if the summed degree days have reached the needed sum of effective
temperatures
}
\seealso{
Other Models:
\code{\link[=calc_degreedays]{calc_degreedays()}},
\code{\link[=calc_mortality]{calc_mortality()}},
\code{\link[=calc_sumefftemp]{calc_sumefftemp()}}
}
\concept{Models}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_degreedays.R
\name{calc_degreedays}
\alias{calc_degreedays}
\title{Calculate degree days}
\usage{
calc_degreedays(x, ts_start, ts_end, ldt, method)
}
\arguments{
\item{x}{SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute}

\item{ts_start}{start of timeseries - Date}

\item{ts_end}{end of timeseries - Date}

\item{ldt}{lower development threshold - numeric}

\item{method}{name of degreedays/temperature sum method - character - either
"baskerville" or "tsum"}
}
\value{
SpatRaster of growing (summed up) degree days.
}
\description{
Calculate degree days
}
\seealso{
Other Models:
\code{\link[=calc_mortality]{calc_mortality()}},
\code{\link[=calc_phenology]{calc_phenology()}},
\code{\link[=calc_sumefftemp]{calc_sumefftemp()}}
}
\concept{Models}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_sumefftemp.R
\name{calc_sumefftemp}
\alias{calc_sumefftemp}
\title{Calculate sum of effective temperatures (SET)}
\usage{
calc_sumefftemp(x, cf_temp, cf_start, cf_end, cf_limit, set, a, b)
}
\arguments{
\item{x}{SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute}

\item{cf_temp}{\itemize{
\item character - "tmean" for cold days and "tmin" for frost days
}}

\item{cf_start}{\itemize{
\item Date - start of cold/frost timeseries
}}

\item{cf_end}{\itemize{
\item Date - end of cold/frost timeseries
}}

\item{cf_limit}{\itemize{
\item numeric - threshold under which a day is defined as
cold/frost day
}}

\item{set}{\itemize{
\item function - x is cold/frost days, a and b are parameter i.e.
function (x, a, b) a + b*x
}}

\item{a}{\itemize{
\item numeric - parameter for set function
}}

\item{b}{\itemize{
\item numeric - parameter for set function
}}
}
\value{
SpatRaster with sum of effective temperatures.
}
\description{
Calculate sum of effective temperatures (SET)
}
\seealso{
Other Models:
\code{\link[=calc_degreedays]{calc_degreedays()}},
\code{\link[=calc_mortality]{calc_mortality()}},
\code{\link[=calc_phenology]{calc_phenology()}}
}
\concept{Models}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_local_daily.R
\docType{data}
\name{local_daily}
\alias{local_daily}
\title{Local daily Weather Station data from Freiburg}
\format{
A data frame with 396 rows and 4 variables
}
\source{
https://opendata.dwd.de/climate_environment/CDC/observations_germany/climate/daily/kl/historical/
}
\description{
A dataset containing daily mean, max and min temperatures for Freiburg from
2019-09-01 to 2020-09-30.
Downloaded from opendata.dwd.de and preprocessed.
}
\details{
The dataset can be loaded using load_test("day").

Stations_id: 01443
Stationsname: Freiburg

The variables are as follows:

\itemize{
\item date in year-month-day - character
\item tmean daily mean temperature in °C - numeric
\item tmax daily max temperature in °C - numeric
\item tmin daily min temperature in °C - numeric
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phenology.R
\name{phenology}
\alias{phenology}
\title{Calculate phenological events}
\usage{
phenology(
  x,
  model,
  parametrisation = NULL,
  year = NULL,
  hatch = NULL,
  return_date = TRUE,
  ...
)
}
\arguments{
\item{x}{SpatRaster list/dataframe (tmean, tmax, tmin) - numeric - with time
attribute/date column}

\item{model}{name of model - character}

\item{parametrisation}{name of parametrisation - character}

\item{year}{year for prognosis - numeric}

\item{hatch}{SpatRaster - logical - with time attribute TRUE/FALSE
hatch/no_hatch}

\item{return_date}{TRUE/FALSE defines output -> see value}

\item{...}{parameter to change default values. (i.e. ldt = 3.5)}
}
\value{
If return_date is TRUE returns single layered SpatRaster with time
serial number (first occurence of phenological event).
If return_date is FALSE returns a one layer per day SpatRaster type logical
with phenological event occurred/not TRUE/FALSE.
}
\description{
Using daily mean or min and max temperature data, the function
calculates the temperature-dependent development stages of OPM or the bud
stages (bud swelling and leaf unfolding) of its host tree
\emph{Quercus robur}.

The default settings correspond to the model described by Halbig et al. 2024.
Additional parametrizations are provided but have not yet been tested.

Halbig et al. 2024
It follows 4 different steps:
\itemize{
\item a) Calculating and summing up cold days or frost days.
(Cold days are defined as days with a mean temperature below ldt
(lower development threshold), while frost days are all days with a min
temperature below ldt).
Hatch dependent development stages need a hatch raster (hatch happened
1 or not 0) for each day
\item b) Calculating degree days with the
single sine method of Baskerville & Emin, 1969 or simple summing up tmean
temperatures over ldt.
\item c) Calculating the needed
sum of effective temperatures for the development stage
\item d) Comparing degree days with the needed sum of effective temperatures
}
}
\examples{
## SpatRaster
srl <- load_test()

# Calculating bud swelling for our raster example
budswelling <- phenology(srl,
                         model = "budswelling",
                         parametrisation = "quercus_robur_clone256_type1",
                         year = 2020)

}
\references{
Halbig et al. 2014: Halbig, P., Stelzer, A. S., Baier, P., Pennerstorfer,
J., Delb, H., & Schopf, A. (2024). PHENTHAUproc–An early warning and decision
support system for hazard assessment and control of oak processionary moth
(Thaumetopoea processionea). Forest Ecology and Management, 552, 121525 \cr
Baskerville & Emin 1969: Baskerville, G. L., & Emin, P. (1969). Rapid
estimation of heat accumulation from maximum and minimum temperatures.
Ecology, 50(3), 514-517. (\url{doi:10.2307/1933912}) \cr
Menzel 1997: Menzel, A. (1997). Phänologie von Waldbäumen unter sich
ändernden Klimabedingungen: Auswertung der Beobachtungen in den
internationalen phänologischen Gärten und Möglichkeiten der Modellierung von
Phänodaten. Frank. \cr
}
\seealso{
Other Main:
\code{\link[=get_legend]{get_legend()}},
\code{\link[=mortality]{mortality()}},
\code{\link[=parameter]{parameter()}},
\code{\link[=phenthau]{phenthau()}}
}
\author{
Bachfischer Lorenz, Department of Forest Protection FVA (2024)
\email{lorenz.bachfischer@posteo.de}
}
\concept{Main}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_station_step.R
\name{plot_station_step}
\alias{plot_station_step}
\title{Plot local PHENTHAUproc in a step plot}
\usage{
plot_station_step(x)
}
\arguments{
\item{x}{output of phenthau - dataframe - local PHENTHAUproc}
}
\value{
A plot showing local PHENTHAUproc results in a step plot.
}
\description{
Plots the development stages of OPM.
}
\examples{

fr_df <- load_test("day")
fr <- phenthau(fr_df)

plot_station_step(fr)
}
\seealso{
Other Plot:
\code{\link[=plot_date]{plot_date()}},
\code{\link[=plot_stages]{plot_stages()}}
}
\concept{Plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_mean.R
\name{calc_mean}
\alias{calc_mean}
\title{Calculate mean TRUE}
\usage{
calc_mean(...)
}
\arguments{
\item{...}{SpatRasters/SpatRaster list - logical - with time attribute}
}
\value{
SpatRaster - logical - with time attribute
}
\description{
Takes multiple logical SpatRasters with time attribute and
returns SpatRaster with the mean time serial number of first and last TRUE
value.
}
\seealso{
Other Calculation:
\code{\link[=calc_first]{calc_first()}},
\code{\link[=calc_last]{calc_last()}}
}
\concept{Calculation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_sr_to_cvec.R
\name{convert_sr_to_cvec}
\alias{convert_sr_to_cvec}
\title{Convert SpatRaster to characer vector}
\usage{
convert_sr_to_cvec(x)
}
\arguments{
\item{x}{SpatRaster - numeric - value is timeserialnumber}
}
\value{
vector - Date as character
}
\description{
Transform local PHENTHAUproc results back from SpatRaster to vector format
}
\seealso{
Other SpatRaster transformation:
\code{\link[=convert_df_to_srl]{convert_df_to_srl()}},
\code{\link[=convert_doy_to_logical]{convert_doy_to_logical()}},
\code{\link[=convert_hour_to_meanminmax]{convert_hour_to_meanminmax()}},
\code{\link[=convert_logical_to_doy]{convert_logical_to_doy()}},
\code{\link[=convert_logical_to_time]{convert_logical_to_time()}}
}
\concept{SpatRaster transformation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter.R
\name{parameter}
\alias{parameter}
\title{Create parameter list}
\usage{
parameter(model = NULL, parametrisation = NULL, year = NULL, first = TRUE)
}
\arguments{
\item{model}{type of model for phenthau function - character - Either single
model or model collection}

\item{parametrisation}{type of parametrisation - character}

\item{year}{year of prognosis - numeric - Default: actual year}

\item{first}{logical - If TRUE and parametrisation is missing first
parametrisation in parameter() is used}
}
\value{
If no argument is specified returns a list of parameter (used inside phenthau
function).
Otherwise returns available parameter for given model, parametrisation and
year
}
\description{
See all available models with parameter: parameter()
model can be a single model or a model collection described in "Default
settings".
Return a data.frame with all model options:
parameter()

\strong{Default Settings}

The default parameter lists for different data input are:
"dailymean": Regional PHENTHAUproc described in Halbig et al. 2024 for daily
mean temperature data
"hour": "Local PHENTHAUproc described in Halbig et al. 2024 for daily hourly
temperature data
"dailymeanminmax": PHENTHAUproc adapted to DWD Data for daily mean, min and
max temperature data

\strong{Columns}

model: model
parametrisation: parametrisation
method: method used to calculate effective temperatures
ts_start: first day to calculate effective temperatures
ts_end last day to calculate effective temperatures (Default 30. Sept)
ts_prevyear: If True calculation of effective temperatures starts in previous
year. (i.e. wagenhoff)
ldt: lower development threshold
cf_dependent: Is model cold/frost dependent
cf_start: first day to calculate cold/frost days
cf_end: last day to calculate cold/frost days
cf_prevyear:  If True calculation of cold/frost days starts in previous year.
cf_temp: tmean" for cold days and "tmin" for frost days
cf_limit: threshold for cold/frost days
set: formula to calculate sum of effective temperatures
a: parameter for set
b: parameter for set
}
\examples{

# Default parameter list for daily mean, min and max temperature data:
parameter("dailymeanminmax")

# overview dataframe with all available parameter sets
parameter("all")

# all hatch model parameter
parameter("hatch")

# return parameter necessary for calculation
parameter("hatch", "custers", 2020)
}
\seealso{
Other Main:
\code{\link[=get_legend]{get_legend()}},
\code{\link[=mortality]{mortality()}},
\code{\link[=phenology]{phenology()}},
\code{\link[=phenthau]{phenthau()}}
}
\concept{Main}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_logical_to_time.R
\name{convert_logical_to_time}
\alias{convert_logical_to_time}
\title{Convert logical to time}
\usage{
convert_logical_to_time(x)
}
\arguments{
\item{x}{SpatRaster - logical - with time attribute}
}
\value{
SpatRaster - numeric
}
\description{
Converts logical SpatRasters to single layer SpatRaster with timeserialnumber as value for first TRUE
}
\seealso{
Other SpatRaster transformation:
\code{\link[=convert_df_to_srl]{convert_df_to_srl()}},
\code{\link[=convert_doy_to_logical]{convert_doy_to_logical()}},
\code{\link[=convert_hour_to_meanminmax]{convert_hour_to_meanminmax()}},
\code{\link[=convert_logical_to_doy]{convert_logical_to_doy()}},
\code{\link[=convert_sr_to_cvec]{convert_sr_to_cvec()}}
}
\concept{SpatRaster transformation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_baskerville.R
\name{calc_baskerville}
\alias{calc_baskerville}
\title{Calculate degree days using the single sine method of Baskerville}
\usage{
calc_baskerville(tmin, tmax, ldt)
}
\arguments{
\item{tmin}{min temperature - numeric}

\item{tmax}{max temperature - numeric}

\item{ldt}{lower development threshold aka base temperature - numeric}
}
\value{
Degree days calculated by using the single sine method of Baskerville.
}
\description{
The function is vectorized.
To use it with SpatRaster data see ?terra::lapp

More Information about the Method itself:
Baskerville, G.L. and P. Emin. 1969. Rapid estimation of heat accumulation
from maximum and minimum temperature. Ecology 50:514-517.
}
\seealso{
Other Temperature sum methods:
\code{\link[=calc_tsum]{calc_tsum()}}
}
\concept{Temperature sum methods}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_time.R
\name{get_time}
\alias{get_time}
\title{Get time attribute}
\usage{
get_time(x)
}
\arguments{
\item{x}{SpatRaster, data.frame, list of SpatRaster or SpatRasterDataset}
}
\value{
If x is a data.frame returns the time column, else it returns the
time attribute of a SpatRaster or the shared time attribute of multiple
SpatRaster.
}
\description{
Get time attribute
}
\seealso{
Other Helper:
\code{\link[=call_function]{call_function()}},
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=convert_dwd_to_phenthau]{convert_dwd_to_phenthau()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=load_test]{load_test()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=subset_time]{subset_time()}},
\code{\link[=timename]{timename()}}
}
\concept{Helper}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality.R
\name{mortality}
\alias{mortality}
\title{Calculating starvation related mortality of \emph{Thaumetopoea processionea}}
\usage{
mortality(
  x,
  par_budswelling = "quercus_robur_clone256_type1",
  par_hatch = c("custers", "wagenhoff", "meurisse"),
  def_hatch = "first",
  last = TRUE
)
}
\arguments{
\item{x}{SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute}

\item{par_budswelling}{character - parametrisation for bud swelling}

\item{par_hatch}{character - parametrisation for hatch}

\item{def_hatch}{definition of hatch - character - either "first" or "mean"}

\item{last}{If TRUE returns only the result for the last day. last row/layer
for data.frame/SpatRaster}
}
\value{
SpatRaster with mortality in \%.
}
\description{
Calculating the starvation-related mortality rate of L1 larvae,
dependent on degree days from the first hatch to feeding start (bud swelling
of the host tree \emph{Quercus robur}).
All already available parameter settings can be shown with parameter().
}
\examples{
\donttest{
srl <- load_test()
mortality(srl)
}
}
\seealso{
Other Main:
\code{\link[=get_legend]{get_legend()}},
\code{\link[=parameter]{parameter()}},
\code{\link[=phenology]{phenology()}},
\code{\link[=phenthau]{phenthau()}}
}
\concept{Main}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timename.R
\name{timename}
\alias{timename}
\title{Create a name with time stamp}
\usage{
timename(x, model, desc = NULL)
}
\arguments{
\item{x}{SpatRaster with time attribute}

\item{model}{character - name of model}

\item{desc}{character - additional description (optional)}
}
\description{
Create a name with time stamp
}
\seealso{
Other Helper:
\code{\link[=call_function]{call_function()}},
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=convert_dwd_to_phenthau]{convert_dwd_to_phenthau()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=get_time]{get_time()}},
\code{\link[=load_test]{load_test()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=subset_time]{subset_time()}}
}
\concept{Helper}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_date.R
\name{plot_date}
\alias{plot_date}
\title{Plot SpatRaster with date serial number/time attribute}
\usage{
plot_date(x, breaks = NULL, ...)
}
\arguments{
\item{x}{SpatRaster - numeric - value is serial number or SpatRaster -
logical - with time attribute}

\item{breaks}{number of breaks in the legend - numeric}

\item{...}{arguments for terra::plot function, except (type, breaks, col or
plg)}
}
\value{
A plot of a phenological event by day with legend.
}
\description{
A wrapper around terra::plot to show the time serial number as
a character date in the legend.
}
\examples{

data <- load_test()
budswelling <- phenology(data, "budswelling", "quercus_robur_clone256_type1", 2020)
plot_date(budswelling)
}
\seealso{
Other Plot:
\code{\link[=plot_stages]{plot_stages()}},
\code{\link[=plot_station_step]{plot_station_step()}}
}
\concept{Plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_first.R
\name{calc_first}
\alias{calc_first}
\title{Calculate first TRUE}
\usage{
calc_first(...)
}
\arguments{
\item{...}{SpatRasters/SpatRaster list - logical - with time attribute}
}
\value{
SpatRaster - logical - with time attribute
}
\description{
Takes multiple logical SpatRasters with time attribute and
returns SpatRaster with common time and TRUE if any SpatRaster is TRUE.
}
\seealso{
Other Calculation:
\code{\link[=calc_last]{calc_last()}},
\code{\link[=calc_mean]{calc_mean()}}
}
\concept{Calculation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_doy_to_logical.R
\name{convert_doy_to_logical}
\alias{convert_doy_to_logical}
\title{Convert day of year (doy) to logical}
\usage{
convert_doy_to_logical(x, from, to, by = "days")
}
\arguments{
\item{x}{SpatRaster - numeric - value is day of year}

\item{from}{Date - ymd - first day of time attribute}

\item{to}{Date - ymd - last day of time attribute}

\item{by}{character - either "days", "weeks" or "months". Reduces return to
one day per day/week/month}
}
\value{
SpatRaster - logical - TRUE/FALSE before_doy/equal_after_doy
}
\description{
Convert day of year (doy) to logical
}
\seealso{
Other SpatRaster transformation:
\code{\link[=convert_df_to_srl]{convert_df_to_srl()}},
\code{\link[=convert_hour_to_meanminmax]{convert_hour_to_meanminmax()}},
\code{\link[=convert_logical_to_doy]{convert_logical_to_doy()}},
\code{\link[=convert_logical_to_time]{convert_logical_to_time()}},
\code{\link[=convert_sr_to_cvec]{convert_sr_to_cvec()}}
}
\concept{SpatRaster transformation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call_function.R
\name{call_function}
\alias{call_function}
\title{Call a function from PHENTHAUproc package}
\usage{
call_function(f, data, params)
}
\arguments{
\item{f}{function}

\item{data}{SpatRaster list (tmean, tmax, tmin) - numeric - with time
attribute}

\item{params}{parameter list}
}
\description{
calls a function from PHENTHAUproc package
}
\seealso{
Other Helper:
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=convert_dwd_to_phenthau]{convert_dwd_to_phenthau()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=get_time]{get_time()}},
\code{\link[=load_test]{load_test()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=subset_time]{subset_time()}},
\code{\link[=timename]{timename()}}
}
\concept{Helper}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_df_to_srl.R
\name{convert_df_to_srl}
\alias{convert_df_to_srl}
\title{Convert dataframe to SpatRaster list}
\usage{
convert_df_to_srl(x, date_col = "date")
}
\arguments{
\item{x}{dataframe (tmean, tmax, tmin, date)}

\item{date_col}{\itemize{
\item character - name for date column which will be used to
create time attribute of SpatRaster.
}}
}
\value{
SpatRaster list (tmean, tmax, tmin) - numeric - with time attribute
}
\description{
Internal function to create a SpatRaster list out of a dataframe.

By default the dataframe should have a date column called "date".
The function is transferring each column into a list object and this object
to a 1 col, 1 row nrow nlyr SpatRaster.
}
\seealso{
Other SpatRaster transformation:
\code{\link[=convert_doy_to_logical]{convert_doy_to_logical()}},
\code{\link[=convert_hour_to_meanminmax]{convert_hour_to_meanminmax()}},
\code{\link[=convert_logical_to_doy]{convert_logical_to_doy()}},
\code{\link[=convert_logical_to_time]{convert_logical_to_time()}},
\code{\link[=convert_sr_to_cvec]{convert_sr_to_cvec()}}
}
\concept{SpatRaster transformation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_logical_to_doy.R
\name{convert_logical_to_doy}
\alias{convert_logical_to_doy}
\title{Convert logical to day of year (doy)}
\usage{
convert_logical_to_doy(x, from = NULL, to = NULL)
}
\arguments{
\item{x}{SpatRaster - logical - with time attribute}

\item{from}{\itemize{
\item Date - ymd - only if SpatRaster should be subset before being
transformed.
}}

\item{to}{\itemize{
\item Date - ymd - only if SpatRaster should be subset before being
transformed.
}}
}
\value{
SpatRaster - numeric - day of year
}
\description{
Converts logical SpatRaster with time attribute and one layer
per day to single layer with the day of the year (doy) where the layer is
1/TRUE first.
}
\seealso{
Other SpatRaster transformation:
\code{\link[=convert_df_to_srl]{convert_df_to_srl()}},
\code{\link[=convert_doy_to_logical]{convert_doy_to_logical()}},
\code{\link[=convert_hour_to_meanminmax]{convert_hour_to_meanminmax()}},
\code{\link[=convert_logical_to_time]{convert_logical_to_time()}},
\code{\link[=convert_sr_to_cvec]{convert_sr_to_cvec()}}
}
\concept{SpatRaster transformation}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_test.R
\name{load_test}
\alias{load_test}
\title{Load test data}
\usage{
load_test(type = "SpatRaster")
}
\arguments{
\item{type}{character, either day/hour/SpatRaster/SpatRaster_hour/budswelling}
}
\value{
data.frame/SpatRaster list with test data
}
\description{
Load test data
}
\seealso{
Other Helper:
\code{\link[=call_function]{call_function()}},
\code{\link[=check_data_with_params]{check_data_with_params()}},
\code{\link[=check_dimension_and_time]{check_dimension_and_time()}},
\code{\link[=convert_dwd_to_phenthau]{convert_dwd_to_phenthau()}},
\code{\link[=get_date]{get_date()}},
\code{\link[=get_formalArgs]{get_formalArgs()}},
\code{\link[=get_time]{get_time()}},
\code{\link[=remove_false]{remove_false()}},
\code{\link[=set_attributes]{set_attributes()}},
\code{\link[=subset_time]{subset_time()}},
\code{\link[=timename]{timename()}}
}
\concept{Helper}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_mortality.R
\name{calc_mortality}
\alias{calc_mortality}
\title{Calculate mortality}
\usage{
calc_mortality(x, budswelling, hatch, mot = 0)
}
\arguments{
\item{x}{SpatRaster list / dataframe with temperature data and time
attribute/column}

\item{budswelling}{SpatRaster - logical - T/F budswelling/no_budswelling}

\item{hatch}{SpatRaster - logical - T/F hatch/no_hatch}

\item{mot}{mortality threshold - numeric - is used as lower development
threshold calculating degree days}
}
\value{
SpatRaster of starving related mortality in \%.
}
\description{
Calculate mortality
}
\seealso{
Other Models:
\code{\link[=calc_degreedays]{calc_degreedays()}},
\code{\link[=calc_phenology]{calc_phenology()}},
\code{\link[=calc_sumefftemp]{calc_sumefftemp()}}
}
\concept{Models}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_stages.R
\name{plot_stages}
\alias{plot_stages}
\title{Plot regional PHENTHAUproc - Stages}
\usage{
plot_stages(x, time = NULL, ...)
}
\arguments{
\item{x}{SpatRaster stages output of phenthau - numeric}

\item{time}{day to plot - Date or character year-month_day i.e.("2020-05-01")}

\item{...}{arguments passed along to terra::plot}
}
\value{
A plot of the PHENTHAUproc stages of the last/chosen time with
preset levels and colors.
}
\description{
A wrapper to plot the development stages of OPM with assigned names and
colors
}
\details{
phenthau returns a list of SpatRasters. The stages object describes
the development stages of oak processionary moth.
This function plots the stages SpatRaster with the right names and colors.
To get IDs, caregories and colors use get_legend("stages").
}
\seealso{
Other Plot:
\code{\link[=plot_date]{plot_date()}},
\code{\link[=plot_station_step]{plot_station_step()}}
}
\concept{Plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_test.R
\name{load_test_hour}
\alias{load_test_hour}
\title{load dataframe - hourly tmean - Freiburg 2023}
\usage{
load_test_hour()
}
\description{
load dataframe - hourly tmean - Freiburg 2023
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_tsum.R
\name{calc_tsum}
\alias{calc_tsum}
\title{Calculate temperature sum}
\usage{
calc_tsum(tmean, ldt)
}
\arguments{
\item{tmean}{numeric - mean temperature}

\item{ldt}{numeric - lower development threshold aka base temperature.}
}
\value{
The temperature sum over the lower development threshold.
}
\description{
The function is vectorized.
To use it with SpatRaster-Data see ?terra::lapp.
}
\seealso{
Other Temperature sum methods:
\code{\link[=calc_baskerville]{calc_baskerville()}}
}
\concept{Temperature sum methods}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_test.R
\name{load_test_bs}
\alias{load_test_bs}
\title{load SpatRaster - budswelling - day of year - 16 Rasterpoints around FVA 2023}
\usage{
load_test_bs()
}
\description{
load SpatRaster - budswelling - day of year - 16 Rasterpoints around FVA 2023
}
\keyword{internal}
